# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

import uuid


class UuidGenerator(object):
    """
    Generator used to generate a universally unique identifier (UUID) string that
    is all lowercase and has enclosing brackets (following McAfee Agent format).
    """

    @staticmethod
    def generate_id():
        """
        Generates and returns a UUID

        :return: The generated UUID
        """
        return uuid.uuid4()

    @staticmethod
    def generate_id_as_string():
        """
        Generates and returns a random UUID that is all lowercase and has enclosing brackets

        :return: A UUID string that is all lowercase and has enclosing brackets
        """
        return "{" + str(UuidGenerator.generate_id()).lower() + "}"

    @staticmethod
    def from_string(string):
        """
        Converts the specified UUID string into a UUID instance

        :param string: The UUID string
        :return: The corresponding UUID instance
        """
        return uuid.UUID(string)

    @staticmethod
    def to_string(uid):
        """
        Converts the specified UUID into string that is all lowercase and has enclosing brackets

        :param uid: The UUID
        :return: A UUID string that is all lowercase and has enclosing brackets
        """
        return "{" + str(uid).lower() + "}"

    @staticmethod
    def normalize(string):
        """
        Normalizes the specified UUID string

        :param string: The UUID string
        :return: A UUID string that is all lowercase and has enclosing brackets
        """
        return UuidGenerator.to_string(UuidGenerator.from_string(string))



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################
from dxlclient import _BaseObject

class DxlUtils(object):
    """
    Utility methods for use by the DXL-related classes
    """

    @staticmethod
    def func_name():
        """
        Returns the current function name
        :return:
        """
        from inspect import currentframe

        return currentframe().f_back.f_code.co_name

    @staticmethod
    def _wildcard_generator(topic):
        """
        Helper method that splits a topic to obtain it's wildcard
        i.e. /foo/bar -> /foo/#
             /foo/bar/ -> /foo/bar/#
             /foo/bar/# -> /foo/#
        :param topic: channel topic.
        :return: wildcarded topic
        """
        if topic is "":
            return "#"
        splitted = topic.split("/")
        if topic[-1] != "#":
            return "/".join(splitted[:-1]) + "/#"
        else:
            if len(topic) is 2:
                return "#"
            return "/".join(splitted[:-2]) + "/#"

    @staticmethod
    def _get_wildcards(topic):
        """
        Helper method that gets a list containing all it's possible wildcards
        :param topic: channel topic
        :return: list containing all channel wildcards
        """
        wildcards = []
        while topic != "#":
            topic = DxlUtils._wildcard_generator(topic)
            wildcards.append(topic)
        return wildcards

    @staticmethod
    def _validate_callback(callback):
        """
        Validates if 'callback' is a valid WildcardCallback

        :param callback: Callback to validate.
        """
        if not issubclass(callback.__class__, WildcardCallback):
            raise ValueError("Type mismatch on callback argument")

    @staticmethod
    def iterate_wildcards(wildcard_callback, topic):
        """
        Iterates the wildcards for the specified topic.
        NOTE: This only supports "#" wildcards (not "+").

        :param wildcard_callback: The callback to invoke for each wildcard
        :param topic: The topic
        :return:
        """

        DxlUtils._validate_callback(wildcard_callback)

        if topic is None:
            return

        topic_wildcards = DxlUtils._get_wildcards(topic)

        for wildcard in topic_wildcards:
            wildcard_callback.on_next_wildcard(wildcard)


class WildcardCallback(_BaseObject):
    """
    Callback that is invoked for each wildcard pattern found
    """

    def on_next_wildcard(self, wildcard):
        """
        Invoked for the next wildcard pattern found

        :param wildcard: The wildcard pattern
        """
        raise NotImplementedError("Must be implemented in a child class.")



import time
from nose.plugins.attrib import attr
from dxlclient import ResponseCallback, UuidGenerator, ServiceRegistrationInfo, Request
from dxlclient.test.base_test import BaseClientTest
from dxlclient.test.test_service import TestService


class AsyncCallbackTimeoutTest(BaseClientTest):

    @attr('manual')
    def test_execute_async_callback_timeout(self):

        # TODO: Set SYSPROP_ASYNC_CALLBACK_CHECK_INTERVAL = 10000 when it is available
        def resp_callback():
            pass

        cb = ResponseCallback()
        cb.on_response = resp_callback

        with self.create_client() as client:
            client.connect()

            req_topic = UuidGenerator.generate_id_as_string()
            missing_topic = UuidGenerator.generate_id_as_string()

            test_service = TestService(client, 1)

            def empty_on_request(request):
                pass

            test_service.on_request = empty_on_request

            reg_info = ServiceRegistrationInfo(client, "async_callback_test_service")
            reg_info.add_topic(req_topic, test_service)
            # Register the service
            client.register_service_sync(reg_info, self.DEFAULT_TIMEOUT)

            async_req = Request(destination_topic=req_topic)
            client.async_request(async_req, cb)  # TODO: Use the method with timeout when is will available

            for i in range(0, 10):
                req = Request(destination_topic=req_topic)
                client.async_request(req, cb)  # TODO: Use the updated method with timeout when it is available

            req_for_error = Request(destination_topic=missing_topic)
            client.async_request(req_for_error)
            async_callback_count = client._get_async_callback_count()
            self.assertEquals(11, async_callback_count)

            for i in range(0, 20):
                print "asyncCallbackCount = " + str(client._get_async_callback_count())
                time.sleep(1)
                req = Request(destination_topic=req_topic)
                client.async_request(req, cb)

            self.assertEquals(1, async_callback_count)

        # TODO: Restore the value of SYSPROP_ASYNC_CALLBACK_CHECK_INTERVAL



import logging

from dxlclient import ErrorResponse, Response
from dxlclient.callbacks import RequestCallback
from concurrent.futures import ThreadPoolExecutor

logger = logging.getLogger(__name__)


class TestService(RequestCallback):

    # The Client
    _client = None
    # Whether to return standard responses or errors
    _return_error = False
    # Thread pool
    _executor = None
    # The error code to return
    _error_code = 99
    # The error message to return
    _error_message = "Error"

    def __init__(self, client, thread_count):
        super(TestService, self).__init__()
        self.m_client = client
        self.m_executor = ThreadPoolExecutor(max_workers=thread_count)

    @property
    def return_error(self):
        return self._return_error

    @return_error.setter
    def return_error(self, return_error):
        self._return_error = return_error

    @property
    def error_code(self):
        return self._error_code

    @error_code.setter
    def error_code(self, error_code):
        self._error_code = error_code

    @property
    def error_message(self):
        return self._error_message

    @error_message.setter
    def error_message(self, error_message):
        self._error_message = error_message

    def on_request(self, request):

        if self._return_error:
            response = ErrorResponse(request, error_code=self._error_code, error_message=self._error_message)
        else:
            response = Response(request)

        def run_task():
            try:
                self.m_client.send_response(response)
            except Exception, e:
                logging.info(e.message)
                raise e

        self.m_executor.submit(run_task)

    def close(self):
        self.m_executor.shutdown()



import logging
import os
import sys

from dxlclient.callbacks import ResponseCallback
from dxlclient.client import DxlClient
from dxlclient.client_config import DxlClientConfig
from dxlclient.message import Message, Request

# Import common logging and configuration
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
from common import *

# Configure local logger
logger = logging.getLogger(__name__)

# Sample topic to fire Requests on
SERVICE_TOPIC = "/isecg/sample/service"

# Sample Synchronous Service Invoker
try:
    # Create DxlClientConfig from expected sample configuration file
    logger.info("Service Invoker - Load DXL config from: %s", CONFIG_FILE)
    config = DxlClientConfig.create_dxl_config_from_file(CONFIG_FILE)

    # Initialize DXL client using our configuration
    logger.info("Service Invoker - Creating DXL Client")
    with DxlClient(config) as client:

        # Connect to DXL Broker
        logger.info("Service Invoker - Connecting to Broker")
        client.connect()

        # Prompt user for input to publish DXL Requests
        while True:
            print "   Press 1 to send a Synchronous Request"
            print "   Press 2 to send an Asynchronous Request"
            print "   Press 9 to quit"
            input = raw_input("   Enter value: ")

            try:
                option = int(input)
            except:
                option = input

            # Option: Synchronous DXL Request
            if option == 1:
                # Create the Request
                logger.info("Service Invoker - Creating Synchronous Request for topic %s", SERVICE_TOPIC)
                request = Request(SERVICE_TOPIC)

                # Encode string payload as UTF-8
                request.payload = ("Sample Synchronous Request Payload - Request ID: " +
                                   str(request.message_id)).encode()

                # Send Synchronous Request with default timeout and wait for Response
                logger.info("Service Invoker - Sending Synchronous Request to %s", SERVICE_TOPIC)
                response = client.sync_request(request)

                # Check that the Response is not an Error Response, then extract
                if response.message_type != Message.MESSAGE_TYPE_ERROR:
                    # Extract information from Response payload, in this sample we expect it is UTF-8 encoded
                    logger.info("Service Invoker - Synchronous Response received:\n   Topic: %s\n   Payload: %s",
                                response.destination_topic,
                                response.payload.decode())
                else:
                    logger.info("Service Invoker - Synchronous Error Response received:\n   Topic: %s\n   Error: %s",
                                response.destination_topic, response.error_message)

            # Option: Asynchronous DXL Request
            elif option == 2:
                # Response callback class to handle DXL Responses from a Service to our Asynchronous Requests
                class MyResponseCallback(ResponseCallback):
                    def on_response(self, response):
                        # Check that the Response is not an Error Response, then extract
                        if response.message_type != Message.MESSAGE_TYPE_ERROR:
                            # Extract information from Response payload, in this sample we expect it is UTF-8 encoded
                            logger.info("Service Invoker - Asynchronous Response received:\n   " +
                                        "Topic: %s\n   Request ID: %s\n   Payload: %s",
                                        response.destination_topic, response.request_message_id,
                                        response.payload.decode())
                        else:
                            logger.info("Service Invoker - Asynchronous Error Response received:\n   " +
                                        "Topic: %s\n   Request ID: %s\n   Error: %s",
                                        response.destination_topic, response.request_message_id, response.error_message)

                # Create the Request
                logger.info("Service Invoker - Creating Asynchronous Request for topic %s", SERVICE_TOPIC)
                request = Request(SERVICE_TOPIC)

                # Encode string payload as UTF-8
                request.payload = 'Sample Asynchronous Request Payload'.encode()

                #Send Asynchronous Request with a timeout of 5 seconds
                logger.info("Service Invoker - Sending Asynchronous Request:\n   Request ID: %s\n   Topic: %s",
                            request.message_id, SERVICE_TOPIC)
                client.async_request(request, MyResponseCallback())

            # Option: Exit the loop
            elif option == 9:
                break

            # Invalid input
            else:
                logger.info("fdsaService Invoker - Invalid input: %s", option)

except Exception as e:
    logger.info("Service Invoker - Exception: %s", e.message)
    exit(1)


# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2014 McAfee Inc. - All Rights Reserved.
################################################################################

# Run with python -m unittest dxlclient.test.test_dxlclient

import unittest
import time
import threading
from base_test import BaseClientTest
import io
from nose.plugins.attrib import attr
from nose_parameterized import parameterized
from mock import Mock, patch
from textwrap import dedent
import __builtin__

import dxlclient._global_settings
from dxlclient import Request
from dxlclient import Response
from dxlclient import Event
from dxlclient import ErrorResponse
from dxlclient import DxlClient
from dxlclient import DxlClientConfig
from dxlclient import Broker
from dxlclient import UuidGenerator
from dxlclient import EventCallback
from dxlclient import RequestCallback
from dxlclient import ResponseCallback
from dxlclient import DxlException
from dxlclient import BrokerListError
from dxlclient._global_settings import *

CONFIG_DATA_NO_CERTS_SECTION = """
[no_certs]
BrokerCertChain: certchain.pem
CertFile: certfile.pem
PrivateKey: privatekey.pk

[Brokers]
22cdcace-6e8f-11e5-29c0-005056aa56de: 22cdcace-6e8f-11e5-29c0-005056aa56de;8883;dxl-broker-1;10.218.73.206
"""
CONFIG_DATA_NO_CA_OPTION = """
[Certs]
CertFile: certfile.pem
PrivateKey: privatekey.pk

[Brokers]
22cdcace-6e8f-11e5-29c0-005056aa56de: 22cdcace-6e8f-11e5-29c0-005056aa56de;8883;dxl-broker-1;10.218.73.206
"""
CONFIG_DATA_NO_CERT_OPTION = """
[Certs]
BrokerCertChain: certchain.pem
PrivateKey: privatekey.pk

[Brokers]
22cdcace-6e8f-11e5-29c0-005056aa56de: 22cdcace-6e8f-11e5-29c0-005056aa56de;8883;dxl-broker-1;10.218.73.206
"""
CONFIG_DATA_NO_PK_OPTION = """
[Certs]
BrokerCertChain: certchain.pem
CertFile: certfile.pem

[Brokers]
22cdcace-6e8f-11e5-29c0-005056aa56de: 22cdcace-6e8f-11e5-29c0-005056aa56de;8883;dxl-broker-1;10.218.73.206
"""
CONFIG_DATA_NO_BROKERS_SECTION = """
[Certs]
BrokerCertChain: certchain.pem
CertFile: certfile.pem
PrivateKey: privatekey.pk

22cdcace-6e8f-11e5-29c0-005056aa56de: 22cdcace-6e8f-11e5-29c0-005056aa56de;8883;dxl-broker-1;10.218.73.206
"""
CONFIG_DATA_NO_BROKERS_OPTION = """
[Certs]
BrokerCertChain: certchain.pem
CertFile: certfile.pem
PrivateKey: privatekey.pk

[Brokers]
"""


class DxlClientConfigTest(unittest.TestCase):
    @parameterized.expand([
        (None,),
        ("",)
    ])
    def test_config_throws_value_error_for_empty_ca_bundle(self, ca_bundle):
        self.assertRaises(ValueError, DxlClientConfig, broker_ca_bundle=ca_bundle,
                          cert_file=get_cert_file_pem(), private_key=get_dxl_private_key(), brokers=[])

    @parameterized.expand([
        (None,),
        ("",)
    ])
    def test_config_throws_value_error_for_empty_cert_file(self, cert_file):
        self.assertRaises(ValueError, DxlClientConfig,
                          cert_file=cert_file, broker_ca_bundle=get_ca_bundle_pem(), private_key=get_dxl_private_key(),
                          brokers=[])

    def test_get_fastest_broker_gets_the_fastest(self):
        semaphore = threading.Semaphore(0)
        # Mock brokers connect speed
        fast_broker = Mock()
        slow_broker = Mock()

        def connect_to_broker_slow():
            import time

            semaphore.acquire()
            time.sleep(0.1)
            return

        def connect_to_broker_fast():
            semaphore.release()
            return

        slow_broker._connect_to_broker = connect_to_broker_slow
        fast_broker._connect_to_broker = connect_to_broker_fast
        # Create config and add brokers
        config = DxlClientConfig(broker_ca_bundle=get_ca_bundle_pem(),
                                 cert_file=get_cert_file_pem(),
                                 private_key=get_dxl_private_key(),
                                 brokers=[])
        config.brokers.append(fast_broker)
        config.brokers.append(slow_broker)
        # Check that the returned is the fastest
        self.assertEqual(config._get_fastest_broker(), fast_broker)

    def test_get_sorted_broker_list_returns_empty_when_no_brokers(self):
        config = DxlClientConfig(broker_ca_bundle=get_ca_bundle_pem(),
                                 cert_file=get_cert_file_pem(),
                                 private_key=get_dxl_private_key(),
                                 brokers=[])
        self.assertEqual(config._get_sorted_broker_list(), [])

    def test_get_sorted_broker_list_returns_all_brokers(self):
        # Create config
        config = DxlClientConfig(broker_ca_bundle=get_ca_bundle_pem(),
                                 cert_file=get_cert_file_pem(),
                                 private_key=get_dxl_private_key(),
                                 brokers=[])
        # Create mocked brokers
        b1 = Mock()
        b2 = Mock()
        b1._connect_to_broker = b2._connect_to_broker = Mock(return_value=True)
        # Add them to config
        config.brokers.append(b1)
        config.brokers.append(b2)
        # Get all brokers
        l = config._get_sorted_broker_list()
        # Check all brokers are in the list
        self.assertTrue(b1 in l)
        self.assertTrue(b2 in l)

    @parameterized.expand([
        ({"BrokersList": "Actually not a brokers list"},)
    ])
    def test_get_brokers_raises_exception_from_invalid_json(self, policy):
        config = DxlClientConfig(broker_ca_bundle=get_ca_bundle_pem(),
                                 cert_file=get_cert_file_pem(),
                                 private_key=get_dxl_private_key(),
                                 brokers=[])
        with self.assertRaises(BrokerListError):
            config._set_brokers_from_json(policy)

    def test_set_config_from_file_generates_dxl_config(self):
        read_data = """
        [Certs]
        BrokerCertChain: certchain.pem
        CertFile: certfile.pem
        PrivateKey: privatekey.pk

        [Brokers]
        22cdcace-6e8f-11e5-29c0-005056aa56de: 22cdcace-6e8f-11e5-29c0-005056aa56de;8883;dxl-broker-1;10.218.73.206
        """

        with patch.object(__builtin__, 'open', return_value=io.BytesIO(dedent(read_data))):
            client_config = DxlClientConfig.create_dxl_config_from_file("mock_file")
            self.assertEqual(client_config.cert_file, "certfile.pem")
            self.assertEqual(client_config.broker_ca_bundle, "certchain.pem")
            self.assertEqual(client_config.private_key, "privatekey.pk")
            broker = client_config.brokers[0]
            self.assertEqual(broker.host_name, "dxl-broker-1")
            self.assertEqual(broker.ip_address, "10.218.73.206")
            self.assertEqual(broker.port, 8883)
            self.assertEqual(broker.unique_id, "22cdcace-6e8f-11e5-29c0-005056aa56de")

    def test_set_config_wrong_file_raises_exception(self):
        with self.assertRaises(Exception):
            DxlClientConfig.create_dxl_config_from_file("this_file_doesnt_exist.cfg")

    @parameterized.expand([
        (CONFIG_DATA_NO_CERTS_SECTION,),
        (CONFIG_DATA_NO_CA_OPTION,),
        (CONFIG_DATA_NO_CERT_OPTION,),
        (CONFIG_DATA_NO_PK_OPTION,),
    ])
    def test_missing_certs_raises_exception(self, read_data):
        with patch.object(__builtin__, 'open', return_value=io.BytesIO(dedent(read_data))):
            with self.assertRaises(ValueError):
                DxlClientConfig.create_dxl_config_from_file("mock_file.cfg")

    @parameterized.expand([
        (CONFIG_DATA_NO_BROKERS_SECTION,),
        (CONFIG_DATA_NO_BROKERS_OPTION,),
    ])
    def test_missing_brokers_doesnt_raise_exceptions(self, read_data):
        with patch.object(__builtin__, 'open', return_value=io.BytesIO(dedent(read_data))):
            client_config = DxlClientConfig.create_dxl_config_from_file("mock_file.cfg")
            self.assertEqual(len(client_config.brokers), 0)


class DxlClientTest(unittest.TestCase):
    def setUp(self):
        self.config = DxlClientConfig(broker_ca_bundle=get_ca_bundle_pem(),
                                      cert_file=get_cert_file_pem(),
                                      private_key=get_dxl_private_key(),
                                      brokers=[])

        mqtt_client_patch = patch('paho.mqtt.client.Client')
        mqtt_client_patch.start()

        self.client = DxlClient(self.config)
        self.client._request_manager.wait_for_response = Mock(return_value=Response(request=None))

        self.test_channel = '/test/channel'

    def tearDown(self):
        patch.stopall()

    def test_client_raises_exception_on_connect_when_already_connecting(self):
        self.client._client.connect.side_effect = Exception("An exception!")

        class MyThread(threading.Thread):
            def __init__(self, client):
                super(MyThread, self).__init__()
                self._client = client

            def run(self):
                self._client.connect()

        t = MyThread(self.client)
        t.setDaemon(True)
        t.start()
        time.sleep(2)

        self.assertEqual(self.client.connected, False)
        with self.assertRaises(DxlException):
            self.client.connect()
            # self.client.disconnect()

    def test_client_raises_exception_on_connect_when_already_connected(self):
        self.client._client.connect.side_effect = Exception("An exception!")
        self.client._connected = Mock(return_value=True)
        with self.assertRaises(DxlException):
            self.client.connect()
            # self.client.disconnect()

    # The following test is too slow
    def test_client_disconnect_doesnt_raises_exception_on_disconnect_when_disconnected(self):
        self.assertEqual(self.client.connected, False)
        self.client.disconnect()
        self.client.disconnect()

    @parameterized.expand([
        # (connect + retries) * 2 = connect_count
        (0, 2),
        (1, 4),
        (2, 6),
    ])
    def test_client_retries_defines_how_many_times_the_client_retries_connection(self, retries, connect_count):
        # Client wont' connect ;)
        self.client._client.connect = Mock(side_effect=Exception('Could not connect'))
        # No delay between retries (faster unit tests)
        self.client.config.reconnect_delay = 0
        self.client._wait_for_policy_delay = 0

        broker = Broker(host_name='localhost')
        broker._parse(UuidGenerator.generate_id_as_string() + ";9999;localhost;127.0.0.1")

        self.client.config.brokers = [broker]
        self.client.config.connect_retries = retries

        with self.assertRaises(DxlException):
            self.client.connect()
        self.assertEqual(self.client._client.connect.call_count, connect_count)
        # self.client.disconnect()

    def test_client_subscribe_adds_subscription_when_not_connected(self):
        self.client._client.subscribe = Mock(return_value=None)
        self.assertFalse(self.client.connected)

        self.client.subscribe(self.test_channel)
        self.assertTrue(self.test_channel in self.client.subscriptions)
        self.assertEqual(self.client._client.subscribe.call_count, 0)

    def test_client_unsubscribe_removes_subscription_when_not_connected(self):
        self.client._client.unsubscribe = Mock(return_value=None)
        self.assertFalse(self.client.connected)
        # Add subscription
        self.client.subscribe(self.test_channel)
        self.assertTrue(self.test_channel in self.client.subscriptions)
        # Remove subscription
        self.client.unsubscribe(self.test_channel)
        self.assertFalse(self.test_channel in self.client.subscriptions)

    def test_client_subscribe_doesnt_add_twice_same_channel(self):
        # Mock client.subscribe and is_connected
        self.client._client.subscribe = Mock(return_value=None)
        self.client._connected = Mock(return_value=True)

        # We always have the default (myself) channel
        self.assertEqual(len(self.client.subscriptions), 1)
        self.client.subscribe(self.test_channel)
        self.assertEqual(len(self.client.subscriptions), 2)
        self.client.subscribe(self.test_channel)
        self.assertEqual(len(self.client.subscriptions), 2)
        self.assertEqual(self.client._client.subscribe.call_count, 1)

    def test_client_handle_message_with_event_calls_event_callback(self):
        event_callback = EventCallback()
        event_callback.on_event = Mock()
        self.client.add_event_callback(self.test_channel, event_callback)
        # Create and process Event
        evt = Event(destination_topic=self.test_channel)._to_bytes()
        self.client._handle_message(self.test_channel, evt)
        # Check that callback was called
        self.assertEqual(event_callback.on_event.call_count, 1)

    def test_client_handle_message_with_request_calls_request_callback(self):
        req_callback = RequestCallback()
        req_callback.on_request = Mock()
        self.client.add_request_callback(self.test_channel, req_callback)
        # Create and process Request
        req = Request(destination_topic=self.test_channel)._to_bytes()
        self.client._handle_message(self.test_channel, req)
        # Check that callback was called
        self.assertEqual(req_callback.on_request.call_count, 1)

    def test_client_handle_message_with_response_calls_response_callback(self):
        callback = ResponseCallback()
        callback.on_response = Mock()
        self.client.add_response_callback(self.test_channel, callback)
        # Create and process Response
        msg = Response(request=None)._to_bytes()
        self.client._handle_message(self.test_channel, msg)
        # Check that callback was called
        self.assertEqual(callback.on_response.call_count, 1)

    def test_client_send_event_publishes_message_to_dxl_fabric(self):
        self.client._client.publish = Mock(return_value=None)
        # Create and process Request
        msg = Event(destination_topic="")
        self.client.send_event(msg)
        # Check that callback was called
        self.assertEqual(self.client._client.publish.call_count, 1)

    def test_client_send_request_publishes_message_to_dxl_fabric(self):
        self.client._client.publish = Mock(return_value=None)
        # Create and process Request
        msg = Request(destination_topic="")
        self.client._send_request(msg)
        # Check that callback was called
        self.assertEqual(self.client._client.publish.call_count, 1)

    def test_client_send_response_publishes_message_to_dxl_fabric(self):
        self.client._client.publish = Mock(return_value=None)
        # Create and process Request
        msg = Response(request=None)
        self.client.send_response(msg)
        # Check that callback was called
        self.assertEqual(self.client._client.publish.call_count, 1)

    def test_client_handles_error_response_and_fire_response_handler(self):
        self.client._fire_response = Mock(return_value=None)
        # Create and process Request
        msg = ErrorResponse(request=None, error_code=666, error_message="test message")
        payload = msg._to_bytes()
        # Handle error response message
        self.client._handle_message(self.test_channel, payload)
        # Check that message response was properly delivered to handler
        self.assertEqual(self.client._fire_response.call_count, 1)

    """
    Service unit tests
    """

    def test_client_register_service_subscribes_client_to_channel(self):
        channel1 = '/mcafee/service/unittest/one'
        channel2 = '/mcafee/service/unittest/two'
        # Create dummy service
        service_info = dxlclient.service.ServiceRegistrationInfo(
            service_type='/mcafee/service/unittest', client=self.client)
        service_info.add_topic(channel1, RequestCallback())
        service_info.add_topic(channel2, RequestCallback())

        # Register service in client
        self.client.register_service_async(service_info)
        # Check subscribed channels
        subscriptions = self.client.subscriptions
        assert channel1 in subscriptions, "Client wasn't subscribed to service channel"
        assert channel2 in subscriptions, "Client wasn't subscribed to service channel"

    def test_client_wont_register_the_same_service_twice(self):
        service_info = dxlclient.service.ServiceRegistrationInfo(
            service_type='/mcafee/service/unittest', client=self.client)

        # Register service in client
        self.client.register_service_async(service_info)
        with self.assertRaises(dxlclient.DxlException):
            # Re-register service
            self.client.register_service_async(service_info)

    def test_client_register_service_sends_register_request_to_broker(self):
        service_info = dxlclient.service.ServiceRegistrationInfo(
            service_type='/mcafee/service/unittest', client=self.client)

        self.client._send_request = Mock(return_value=True)
        self.client._connected = Mock(return_value=True)

        # Register service in client
        self.client.register_service_async(service_info)
        time.sleep(2)
        # Check that method has been called
        self.assertTrue(self.client._send_request.called)

    def test_client_register_service_unsubscribes_client_to_channel(self):
        channel1 = '/mcafee/service/unittest/one'
        channel2 = '/mcafee/service/unittest/two'
        # Create dummy service
        service_info = dxlclient.service.ServiceRegistrationInfo(
            service_type='/mcafee/service/unittest', client=self.client)
        service_info.add_topic(channel1, RequestCallback())
        service_info.add_topic(channel2, RequestCallback())

        # Register service in client
        self.client.register_service_async(service_info)
        # Check subscribed channels
        subscriptions = self.client.subscriptions
        assert channel1 in subscriptions, "Client wasn't subscribed to service channel"
        assert channel2 in subscriptions, "Client wasn't subscribed to service channel"

        self.client.unregister_service_async(service_info)
        subscriptions = self.client.subscriptions
        assert channel1 not in subscriptions, "Client wasn't unsubscribed to service channel"
        assert channel2 not in subscriptions, "Client wasn't unsubscribed to service channel"

    def test_client_register_service_unsuscribes_from_channel_by_guid(self):
        channel1 = '/mcafee/service/unittest/one'
        channel2 = '/mcafee/service/unittest/two'

        # Create dummy service
        service_info = dxlclient.service.ServiceRegistrationInfo(
            service_type='/mcafee/service/unittest', client=self.client)
        service_info.add_topic(channel1, RequestCallback())
        service_info.add_topic(channel2, RequestCallback())

        # Create same dummy service - different object
        service_info2 = service_info = dxlclient.service.ServiceRegistrationInfo(
            service_type='/mcafee/service/unittest', client=self.client)
        service_info._service_id = service_info.service_id
        service_info.add_topic(channel1, RequestCallback())
        service_info.add_topic(channel2, RequestCallback())

        # Register service in client
        self.client.register_service_async(service_info)

        # Check subscribed channels
        subscriptions = self.client.subscriptions
        assert channel1 in subscriptions, "Client wasn't subscribed to service channel"
        assert channel2 in subscriptions, "Client wasn't subscribed to service channel"

        self.client.unregister_service_async(service_info2)
        subscriptions = self.client.subscriptions
        assert channel1 not in subscriptions, "Client wasn't unsubscribed to service channel"
        assert channel2 not in subscriptions, "Client wasn't unsubscribed to service channel"


@attr('system')
class DxlClientSystemClientTest(BaseClientTest):

    def test_client_connects_to_broker_and_sets_current_broker(self):

        with self.create_client() as client:
            client.connect()
            broker_id = "unique_broker_id_1"

            self.assertTrue(client.connected)
            self.assertEqual(client.current_broker.unique_id, broker_id)

    def test_client_raises_exception_when_cannot_sync_connect_to_broker(self):

        with self.create_client() as client:
            broker = Broker("localhost", UuidGenerator.generate_id_as_string(), "127.0.0.1")
            client._config.brokers = [broker]

            with self.assertRaises(DxlException):
                client.connect()

    def test_client_receives_event_on_topic_only_after_subscribe(self):
        """
        The idea of this test is to send an event to a topic which we are not
        subscribed, so we shouldn't be notified. Then, we subscribe to that
        topic and send a new event, we should get that last one.
        """
        with self.create_client() as client:
            test_topic = '/test/whatever/' + client.config._client_id
            client.connect()
            time.sleep(2)
            self.assertTrue(client.connected)

            # Set request callback (use mock to easily check when it was called)
            ecallback = EventCallback()
            ecallback.on_event = Mock()
            client.add_event_callback(test_topic, ecallback, False)

            # Send event thru dxl fabric to a topic which we are *not* subscribed
            msg = Event(destination_topic=test_topic)
            client.send_event(msg)

            time.sleep(1)
            # We haven't been notified
            self.assertEqual(ecallback.on_event.call_count, 0)

            # Subscribe to topic
            client.subscribe(test_topic)
            time.sleep(1)

            # Send event thru dxl fabric again to that topic
            msg = Event(destination_topic=test_topic)
            client.send_event(msg)

            time.sleep(1)
            # Now we should have been notified of the event
            self.assertEqual(ecallback.on_event.call_count, 1)

    def test_client_receives_error_response_on_request_to_unknown_service(self):
        """
        The idea of this test is to send a sync request to an unknown service
        and get a "unable to locate service" error response.
        """
        with self.create_client() as client:
            test_topic = '/test/doesntexists/' + client.config._client_id
            client.connect()
            time.sleep(2)
            self.assertTrue(client.connected)

            # Send request thru dxl fabric to a service which doesn't exists
            msg = Request(destination_topic=test_topic)
            msg.service_id = UuidGenerator.generate_id_as_string()
            response = client.sync_request(msg, 1)

            # Check that we have an error response for our request
            self.assertTrue(isinstance(response, ErrorResponse))
            self.assertEqual(response.service_id, msg.service_id)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################
__version__ = "3.0.1.203"

__product_id__ = "DXL_____1000"

__product_name__ = "McAfee Data Exchange Layer"

__product_props__ = {
    "General":
        {
            "Version": __version__,
            "ProductName": __product_name__,
            "Language": "0000"
        }
}

def get_product_id():
    """
    Returns DXL Client product ID.

    :returns: {@code string}: Product ID.
    """
    return __product_id__


def get_product_version():
    """
    Returns DXL Client version.

    :returns: {@code string}: version.
    """
    return __version__


def get_product_name():
    """
    Returns DXL Client product name.

    :returns: {@code string}: product name.
    """
    return __product_name__


def get_product_props():
    """
    Returns DXL Client properties.

    :returns: {@code dict}: Properties of the client..
    """
    return __product_props__



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2014 McAfee Inc. - All Rights Reserved.
################################################################################

# Run with python -m unittest dxlclient.test.test_message

import unittest

from dxlclient import Message
from dxlclient import Event
from dxlclient import Request
from dxlclient import Response
from dxlclient import ErrorResponse
from dxlclient import UuidGenerator

from pprint import PrettyPrinter

#
# Configure pretty printer
#
pp = PrettyPrinter(indent=2, width=120)


class MessageTest(unittest.TestCase):
    def setUp(self):
        pass

    def test_event(self):
        source_client_guid = UuidGenerator.generate_id_as_string()
        source_broker_guid = UuidGenerator.generate_id_as_string()

        event = Event(destination_topic="")
        event._source_client_id = source_client_guid
        event._source_broker_id = source_broker_guid
        event.broker_ids = ["{66000000-0000-0000-0000-000000000001}",
                            "{66000000-0000-0000-0000-000000000002}",
                            "{66000000-0000-0000-0000-000000000003}"]
        event.client_ids = ["{25000000-0000-0000-0000-000000000001}",
                            "{25000000-0000-0000-0000-000000000002}",
                            "{25000000-0000-0000-0000-000000000003}"]
        event.payload = str.encode("EVENT")

        pp.pprint(vars(event))
        message = event._to_bytes()
        pp.pprint(message)

        result = Message._from_bytes(message)
        pp.pprint(vars(result))

        assert result.source_client_id == source_client_guid
        assert result.source_broker_id == source_broker_guid
        assert result.broker_ids == ["{66000000-0000-0000-0000-000000000001}",
                                     "{66000000-0000-0000-0000-000000000002}",
                                     "{66000000-0000-0000-0000-000000000003}"]
        assert result.client_ids == ["{25000000-0000-0000-0000-000000000001}",
                                     "{25000000-0000-0000-0000-000000000002}",
                                     "{25000000-0000-0000-0000-000000000003}"]
        assert result.payload == str.encode("EVENT")
        assert result.message_type == Message.MESSAGE_TYPE_EVENT

    def test_event_with_empty_broker_and_client_guids(self):
        source_client_guid = UuidGenerator.generate_id_as_string()
        source_broker_guid = UuidGenerator.generate_id_as_string()

        event = Event(destination_topic="")
        event._source_client_id = source_client_guid
        event._source_broker_id = source_broker_guid
        event.payload = str.encode("EVENT")

        pp.pprint(vars(event))
        message = event._to_bytes()
        pp.pprint(message)

        result = Message._from_bytes(message)
        pp.pprint(vars(result))

        assert isinstance(result.broker_ids, list)
        assert isinstance(result.client_ids, list)

    def test_request(self):
        reply_to_channel = "/mcafee/client/" + UuidGenerator.generate_id_as_string()
        service_guid = UuidGenerator.generate_id_as_string()
        source_client_guid = UuidGenerator.generate_id_as_string()
        source_broker_guid = UuidGenerator.generate_id_as_string()

        request = Request(destination_topic="")
        request.reply_to_topic = reply_to_channel
        request.service_id = service_guid
        request._source_client_id = source_client_guid
        request._source_broker_id = source_broker_guid
        request.broker_ids = ["{66000000-0000-0000-0000-000000000001}",
                              "{66000000-0000-0000-0000-000000000002}",
                              "{66000000-0000-0000-0000-000000000003}"]
        request.client_ids = ["{25000000-0000-0000-0000-000000000001}",
                              "{25000000-0000-0000-0000-000000000002}",
                              "{25000000-0000-0000-0000-000000000003}"]
        request.payload = str.encode("REQUEST")

        pp.pprint(vars(request))
        message = request._to_bytes()
        pp.pprint(message)

        result = Message._from_bytes(message)
        pp.pprint(vars(result))

        assert result.reply_to_topic == reply_to_channel
        assert result.service_id == service_guid
        assert result.source_client_id == source_client_guid
        assert result.source_broker_id == source_broker_guid
        assert result.broker_ids == ["{66000000-0000-0000-0000-000000000001}",
                                     "{66000000-0000-0000-0000-000000000002}",
                                     "{66000000-0000-0000-0000-000000000003}"]
        assert result.client_ids == ["{25000000-0000-0000-0000-000000000001}",
                                     "{25000000-0000-0000-0000-000000000002}",
                                     "{25000000-0000-0000-0000-000000000003}"]
        assert result.payload == str.encode("REQUEST")
        assert result.message_type == Message.MESSAGE_TYPE_REQUEST

    def test_request_and_response(self):
        reply_to_channel = "/mcafee/client/" + UuidGenerator.generate_id_as_string()
        service_guid = UuidGenerator.generate_id_as_string()
        source_client_guid = UuidGenerator.generate_id_as_string()
        source_broker_guid = UuidGenerator.generate_id_as_string()

        request = Request(destination_topic="")
        request.reply_to_channel = reply_to_channel
        request.service_id = service_guid
        request._source_client_id = source_client_guid
        request._source_broker_id = source_broker_guid
        request.broker_ids = ["{66000000-0000-0000-0000-000000000001}",
                              "{66000000-0000-0000-0000-000000000002}",
                              "{66000000-0000-0000-0000-000000000003}"]
        request.client_ids = ["{25000000-0000-0000-0000-000000000001}",
                              "{25000000-0000-0000-0000-000000000002}",
                              "{25000000-0000-0000-0000-000000000003}"]
        request.payload = str.encode("REQUEST")

        pp.pprint(vars(request))
        message = request._to_bytes()
        pp.pprint(message)

        assert request.source_client_id == source_client_guid

        result = Message._from_bytes(message)
        pp.pprint(vars(result))

        response = Response(request=request)
        response.payload = str.encode("RESPONSE")

        pp.pprint(vars(response))
        message = response._to_bytes()
        pp.pprint(message)
        result = Message._from_bytes(message)
        pp.pprint(vars(result))

        assert result.message_type == Message.MESSAGE_TYPE_RESPONSE

    def test_error_response(self):
        error_code = 99
        error_message = "This is an error"

        response = ErrorResponse(request=None,
                                 error_code=error_code,
                                 error_message=error_message)

        pp.pprint(vars(response))
        message = response._to_bytes()
        pp.pprint(message)

        result = Message._from_bytes(message)
        pp.pprint(vars(result))

        assert result.error_code == error_code
        assert result.error_message == error_message
        assert result.message_type == Message.MESSAGE_TYPE_ERROR



from dxlclient.test.base_test import BaseClientTest
from dxlclient import Request, ErrorResponse
from nose.plugins.attrib import attr

@attr('system')
class BrokerRegistryQueryTest(BaseClientTest):

    #
    # Tests Test the broker registry query.
    #
    @attr('system')
    def test_execute_registry_query(self):
        with self.create_client() as client:
            client.connect()
            topic = "/mcafee/service/dxl/brokerregistry/query"

            req = Request(topic)
            req.payload = "{}"
            response = client.sync_request(req)

            self.assertNotIsInstance(response, ErrorResponse)
            print "## sourceBrokerGuid: " + str(response.source_broker_id)
            print "## sourceClientGuid: " + str(response.source_client_id)
            print str(response.payload)



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

import re

# This regex matches dotted-quad IPv4 addresses, like 123.123.123.123
REGEX_V4ADDR = re.compile(
    r"^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$")
# This monster matches all valid IPv6 addresses, including the embedded dotted-quad notation
# pylint: disable=line-too-long
REGEX_V6ADDR = re.compile(
    r"^(^(([0-9A-Fa-f]{1,4}(((:[0-9A-Fa-f]{1,4}){5}::[0-9A-Fa-f]{1,4})|((:[0-9A-Fa-f]{1,4}){4}::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,1})|((:[0-9A-Fa-f]{1,4}){3}::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,2})|((:[0-9A-Fa-f]{1,4}){2}::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,3})|(:[0-9A-Fa-f]{1,4}::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,4})|(::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,5})|(:[0-9A-Fa-f]{1,4}){7}))$|^(::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,6})$)|^::$)|^(^(([0-9A-Fa-f]{1,4}((((:[0-9A-Fa-f]{1,4}){4}:)|(:[0-9A-Fa-f]{1,4}){3}:(:[0-9A-Fa-f]{1,4}){0,1})|((:[0-9A-Fa-f]{1,4}){2}:(:[0-9A-Fa-f]{1,4}){0,2})|((:[0-9A-Fa-f]{1,4}):(:[0-9A-Fa-f]{1,4}){0,3})|(:(:[0-9A-Fa-f]{1,4}){0,4})|(:[0-9A-Fa-f]{1,4}){5}))|^(::[0-9A-Fa-f]{1,4}(:[0-9A-Fa-f]{1,4}){0,4}))|^:):((25[0-5]|2[0-4][0-9]|1[0-9]{2}|[0-9][0-9]|[0-9])\.){3}(25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{0,2})$")
# This regex matches NetBIOS names (see http://support.microsoft.com/kb/909264)
REGEX_NETBIOS = re.compile(r"^([^\\/:*?\"<>|.]{1,15})$")
# This regex matches DNS names (see http://support.microsoft.com/kb/909264)
REGEX_FQDN = re.compile(
    r"^(([a-zA-Z0-9]|[a-zA-Z0-9][^\s,~:!@#$%\^&'\.\(\)\{\}_]*[^\s,~:!@#$%\^&'\.\(\)\{\}_\-])\.)*([a-zA-Z0-9]|[a-zA-Z0-9][^\s,~:!@#$%\^&'\.\(\)\{\}_]*[^\s,~:!@#$%\^&'\.\(\)\{\}_\-])$")


class ServerNameHelper(object):
    @staticmethod
    def is_valid_ip_address(ip_address):
        """
        Returns {@code True} if `ip_address` is a valid ipv4 or ipv6 ip address.

        :param ip_address: Address to validate.
        """
        return (ip_address is not None and ip_address != "" and (
            ServerNameHelper.is_valid_ipv4_address(ip_address) or ServerNameHelper.is_valid_ipv6_address(ip_address)))

    @staticmethod
    def is_valid_ipv4_address(ip_address):
        """
        Returns {@code True} if `ip_address` is a valid ipv4 ip address.

        :param ip_address: Address to validate.
        """
        return ip_address is not None and ip_address != "" and REGEX_V4ADDR.match(ip_address)

    @staticmethod
    def is_valid_ipv6_address(ip_address):
        """
        Returns {@code True} if `ip_address` is a valid ipv6 ip address.

        :param ip_address: Address to validate.
        """
        return ip_address is not None and ip_address != "" and REGEX_V6ADDR.match(ip_address)

    @staticmethod
    def is_valid_netbios_name(hostname):
        """
        Returns {@code True} if `hostname` is a valid NETBIOS name.

        :param hostname: Hostname to validate.
        """
        return hostname is not None and hostname != "" and REGEX_NETBIOS.match(hostname)

    @staticmethod
    def is_valid_fqdn(hostname):
        """
        Returns {@code True} if `hostname` is a valid FQDN.

        :param hostname: Hostname to validate.
        """
        return hostname is not None and hostname != "" and REGEX_FQDN.match(hostname)

    @staticmethod
    def is_valid_hostname_or_ipv4_address(hostname):  # pylint: disable=invalid-name
        """
        Returns {@code True} if `hostname` is a valid FQDN or a valid ipv4 address.

        :param hostname: Hostname to validate.
        """
        return (ServerNameHelper.is_valid_ipv4_address(hostname) or ServerNameHelper.is_valid_netbios_name(
            hostname) or ServerNameHelper.is_valid_fqdn(hostname))

    @staticmethod
    def is_valid_hostname_or_ip_address(hostname):
        """
        Returns {@code True} if `hostname` is a valid FQDN or a valid ipv4/ipv6 address.

        :param hostname: Hostname to validate.
        """
        return (ServerNameHelper.is_valid_ipv4_address(hostname) or ServerNameHelper.is_valid_ipv6_address(
            hostname) or ServerNameHelper.is_valid_netbios_name(hostname) or ServerNameHelper.is_valid_fqdn(hostname))




from base_test import BaseClientTest
from threading import RLock, Condition
from nose.plugins.attrib import attr
import time
import logging
import weakref
import gc

from dxlclient import Response, DxlException, RequestCallback, ServiceRegistrationInfo
from dxlclient import EventCallback
from dxlclient.service import _ServiceManager
from dxlclient import Request
from dxlclient.exceptions import WaitTimeoutException
from dxlclient import ErrorResponse


class EventCounterCallback(EventCallback):

    counter_lock = RLock()
    condition = Condition()
    counter = 0

    def get(self):
        with self.counter_lock:
            return self.counter

    def reset(self):
        with self.condition:
            with self.counter_lock:
                self.counter = 0
            self.condition.notify_all()

    def on_event(self, event):
        print event.destination_topic
        print event.payload

        with self.counter_lock:
            with self.condition:
                self.counter += 1
                self.condition.notify_all()


@attr('system')
class RegisterServiceClientTest(BaseClientTest):

    register_callback = EventCounterCallback()
    unregister_callback = EventCounterCallback()

    info = None
    request_callback = None

    def add_client_callbacks(self, client):
        self.request_callback = RequestCallback()

        def on_request(request):
            logging.info(request.destination_topic)
            logging.info(request.payload)

            response = Response(request)
            response.payload = bytes("Ok")
            try:
                client.send_response(response)
            except DxlException, ex:
                print "Failed to send response" + str(ex)

        self.request_callback.on_request = on_request

        client.add_event_callback(_ServiceManager.DXL_SERVICE_REGISTER_CHANNEL, self.register_callback)
        client.add_event_callback(_ServiceManager.DXL_SERVICE_UNREGISTER_CHANNEL, self.unregister_callback)

        self.info = ServiceRegistrationInfo(client, "/mcafee/service/JTI")
        self.info.add_topic("/mcafee/service/JTI/file/reputation/" + self.info.service_id, self.request_callback)
        self.info.add_topic("/mcafee/service/JTI/cert/reputation/" + self.info.service_id, self.request_callback)

    def setUp(self):
        super(RegisterServiceClientTest, self).setUp()

        self.register_callback.reset()
        self.unregister_callback.reset()

    @attr('system')
    def test_register_service_before_connect(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            client.register_service_async(self.info)
            client.connect()
            time.sleep(self.POST_OP_DELAY)
            client.unregister_service_sync(self.info, self.REG_DELAY)

            self.assertEquals(1, self.register_callback.get())
            self.assertEquals(1, self.unregister_callback.get())

    @attr('system')
    def test_register_service_after_connect(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            client.connect()
            client.register_service_sync(self.info, self.REG_DELAY)
            client.unregister_service_sync(self.info, self.REG_DELAY)

            self.assertEquals(1, self.register_callback.get())
            self.assertEquals(1, self.unregister_callback.get())

    @attr('system')
    def test_register_service_never_connect(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            client.register_service_async(self.info)

            client.unregister_service_async(self.info)

            self.assertEquals(0, self.register_callback.get())
            self.assertEquals(0, self.unregister_callback.get())

    @attr('system')
    def test_register_unregister_service_before_connect(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            client.register_service_async(self.info)
            client.unregister_service_async(self.info)
            client.connect()

            self.assertEquals(0, self.register_callback.get())
            self.assertEquals(0, self.unregister_callback.get())

    @attr('system')
    def test_register_service_and_send_request(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            client.register_service_async(self.info)
            time.sleep(self.POST_OP_DELAY)
            client.connect()
            time.sleep(self.POST_OP_DELAY)

            request = Request("/mcafee/service/JTI/file/reputation/" + self.info.service_id)
            request.payload = bytes("Test")

            response = client.sync_request(request, self.POST_OP_DELAY)
            logging.info("Response payload: {0}".format(str(response.payload)))

            self.assertEquals("Ok", str(response.payload))

    @attr('system')
    def test_register_service_weak_reference_before_connect(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            ref = weakref.ref(self.info)

            client.register_service_async(self.info)

            # Deleted the service registration
            self.info = 1
            time.sleep(self.POST_OP_DELAY * 2)

            # Enforce garbage collection
            gc.collect()
            # Weak reference should now be null
            self.assertEquals(None, ref())

            client.connect()

        self.assertEquals(0, self.register_callback.get())
        self.assertEquals(0, self.unregister_callback.get())

    @attr('system')
    def test_register_service_weak_reference_after_connect(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            ref = weakref.ref(self.info)
            client.register_service_async(self.info)

            client.connect()

            # Deleted the service registration
            self.info = 1
            time.sleep(self.POST_OP_DELAY)

            # Enforce garbage collection
            gc.collect()
            # Weak reference should now be null
            self.assertEquals(None, ref())

        self.assertEquals(1, self.register_callback.get())

        #
        # Sometimes the unregister event does not get send; don't check for now
        # self.assertEquals(1, self.unregister_callback.get())
        #

    @attr('system')
    def test_register_service_weak_reference_after_connect_and_send_request(self):

        with self.create_client() as client:
            self.add_client_callbacks(client)
            ref = weakref.ref(self.info)

            client.register_service_async(self.info)
            client.connect()

            info_guid = self.info.service_id
            # Theoretically, sending the destroy method when creating the weakref
            # would made the magic of calling that method when info get unref
            self.info._destroy()
            # Deleted the service registration
            self.info = 1
            time.sleep(self.POST_OP_DELAY * 2)

            # Enforce garbage collection
            gc.collect()
            # Weak reference should now be null
            self.assertEquals(None, ref())

            # Sending an request should result in a WaitTimeoutException since the destroy() method
            # of ServiceRegistrationInfo will unregister the service
            request = Request("/mcafee/service/JTI/file/reputation/" + info_guid)
            request.payload = bytes("Test")

            try:
                response = client.sync_request(request, 2)
                # Depending upon the timing, the broker can respond with 404 or the request might timeout
                # self.assertIsInstance(response, ErrorResponse, "response is instance of ErrorResponse")
                self.assertTrue(isinstance(response, ErrorResponse), response.__class__)
            except WaitTimeoutException as ex:
                assert(ex.message.__contains__(request.message_id))

        self.assertEquals(1, self.register_callback.get())
        logging.debug("Waiting for unregister event...")
        ttw = 30
        while self.unregister_callback.get() < 1 and ttw > 0:
            time.sleep(self.POST_OP_DELAY)
            ttw -= 1
        # Sometimes the unregister event does not get send; don't check for now
        # self.assertEquals(1, self.unregister_callback.get())



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

import re
import socket
import datetime
import logging

from dxlclient import _BaseObject
from dxlclient.exceptions import MalformedBrokerUriException
from dxlclient._server_name_helper import ServerNameHelper

logger = logging.getLogger(__name__)


class Broker(_BaseObject):
    """
    The class :class:`Broker` represents a DXL message broker.

    Instances of this class are created for the purpose of connecting to the DXL fabric.

    There are several ways to create :class:`Broker` instances:

    - Invoking the :class:`Broker` constructor directly
    - Passing a properly formatted string to the static :func:`parse` method of :class:`Broker`
    - When creating a :class:`dxlclient.client_config.DxlClientConfig` object
      via the :func:`dxlclient.client_config.DxlClientConfig.create_dxl_config_from_file` static method

    """
    _FIELD_SEPARATOR = ';'
    """The field separator used by the e :func:`dxlclient.broker.Broker.parse` method"""
    _SSL_PROTOCOL = "ssl"
    """The SSL protocol identifier"""
    _SSL_PORT = 8883
    """The standard TLS port"""

    def __init__(self, host_name, unique_id=None, ip_address=None, port=_SSL_PORT):
        """
        Constructor parameters:

        :param host_name: The host name or IP address of the broker (required)
        :param unique_id: A unique identifier for the broker, used to identify the broker in log messages, etc.
            (optional)
        :param ip_address: A valid IP address for the broker. This allows for both the host name and IP address
            to be used when connecting to the broker (optional).
        :param port: The port of the broker (defaults to 8883)
        """
        super(Broker, self).__init__()

        # The unique ID of the Broker
        self.unique_id = unique_id
        # The hostName or IP address of the broker
        self.host_name = host_name
        # The IP address of the broker (optional)
        self.ip_address = ip_address
        # The TCP port of the broker
        self.port = port
        # The broker response came from the secondary test using the IP address
        self._response_from_ip_address = None
        # The broker response time in nanoseconds, or None if no response or not tested
        self._response_time = None

    def __del__(self):
        """Destructor"""
        super(Broker, self).__del__()

    @property
    def unique_id(self):
        """
        A unique identifier for the broker, used to identify the broker in log messages, etc.
        """
        return self._unique_id

    @unique_id.setter
    def unique_id(self, id):
        if id:
            self._unique_id = id
        else:
            self._unique_id = ""

    @property
    def host_name(self):
        """
        The host name or IP address of the broker
        """
        return self._host_name

    @host_name.setter
    def host_name(self, host_name):
        # Remove brackets around IPv6 address
        host = re.sub(r"[\[\]]", "", host_name)
        if not ServerNameHelper.is_valid_hostname_or_ip_address(host):
            raise MalformedBrokerUriException("Invalid host name")
        self._host_name = host

    @property
    def ip_address(self):
        """
        A valid IP address for the broker. This allows for both the host name and IP address
        to be used when connecting to the broker
        """
        return self._ip_address

    @ip_address.setter
    def ip_address(self, ip_address):
        ip = ip_address
        if ip:
            # Remove brackets around IPv6 address
            ip = re.sub(r"[\[\]]", "", ip_address)
            if not ServerNameHelper.is_valid_ip_address(ip):
                raise MalformedBrokerUriException("Invalid IP address")
        self._ip_address = ip

    @property
    def port(self):
        """
        The port of the broker
        """
        return self._port

    @port.setter
    def port(self, port):
        if Broker._is_port_number(port):
            self._port = int(port)
        else:
            raise MalformedBrokerUriException("Invalid port")

    def to_string(self):
        """Returns a string representation of the broker for the purposes of logging, etc."""
        ret = "{"
        if self.unique_id:
            ret += "Unique id: " + self.unique_id + ", "
        ret += "Host name: " + self.host_name
        if self.ip_address:
            ret += ", IP address: " + self.ip_address
        ret += ", Port: " + str(self.port)
        ret += "}"
        return ret

    @staticmethod
    def parse(broker_url):
        """
        Returns a broker instance corresponding to the specified broker URL of the form:

        ``[ssl://]<hostname>[:port]``

        Valid URLs include:

        - ``ssl://mybroker:8883``
        - ``ssl://mybroker``
        - ``mybroker:8883``
        - ``mybroker``

        If the port is omitted it will be defaulted to 8883.

        :param broker_url: A valid broker URL
        :return: A broker corresponding to the specified broker URL
        """
        broker = Broker(host_name='none')
        elements = broker_url.split("://")
        host_name = broker_url
        protocol = Broker._SSL_PROTOCOL
        port = Broker._SSL_PORT
        if len(elements) == 2:
            protocol = elements[0]
            host_name = elements[1]
        elements = host_name.split(":")
        if len(elements) == 2:
            host_name = elements[0]
            port = elements[1]
        broker.host_name = host_name
        broker.port = port

        if protocol and not protocol.lower() == Broker._SSL_PROTOCOL.lower():
            raise MalformedBrokerUriException("Unknown protocol: " + protocol)

        return broker

    def _parse(self, broker_string):
        """
        Constructs a Broker object for the specified broker string
        in the format [UniqueId];[Port];[HostName];[IpAddress]

        :param broker_string: The broker definition as a string
        :return: None
        """
        elements = [a.strip() for a in broker_string.split(self._FIELD_SEPARATOR)]
        len_elems = len(elements)
        attrs = {}
        if len_elems < 2:
            raise MalformedBrokerUriException("Missing elements")
        else:
            if Broker._is_port_number(elements[0]):
                attrs['port'] = elements[0]
                attrs['hostname'] = elements[1]
                if len_elems > 2:
                    attrs['ip_address'] = elements[2]
            else:
                attrs['unique_id'] = elements[0]
                attrs['port'] = elements[1]
                attrs['hostname'] = elements[2]
                if len_elems > 3:
                    attrs['ip_address'] = elements[3]
        # Sets broker attributes from the dictionary generated.
        self._set_attributes(attrs)

    def _connect_to_broker(self):
        """
        Attempts to connect to a broker.
        Upon success will set the response time and whether the response
        came from the IP address versus the hostname.
        :return: None
        """
        broker_s = None
        try:
            start = datetime.datetime.now()
            broker_s = socket.create_connection((self._host_name, self._port), timeout=1.0)
            end = datetime.datetime.now()
            self._response_from_ip_address = False
            self._response_time = (end - start).total_seconds()
        except socket.error, msg:
            if self._ip_address is not None:
                try:
                    start = datetime.datetime.now()
                    broker_s = socket.create_connection((self._ip_address, self._port), timeout=1.0)
                    end = datetime.datetime.now()
                    self._response_from_ip_address = True
                    self._response_time = (end - start).total_seconds()
                except socket.error, msg:
                    logger.error("Socket could not be created. Error Code : " + str(msg.errno) + " Message " + str(msg.message))
            else:
                logger.error("Socket could not be created. Error Code : " + str(msg.errno) + " Message " + str(msg.message))
        finally:
            if broker_s is not None:
                broker_s.close()

    def _set_attributes(self, attrs):
        """
        Sets broker attributes from the input dictionary with format:
            {'unique_id': <UNIQUE_ID>, 'port': <PORT>, 'hostname': <HOSTNAME>, 'ip_address': <IP>}
            Required keys: 'port' and 'hostname'.
        Pre: 'port' in attrs and 'hostname' in attrs
        :param attrs: input dictionary with broker attributes.
        """
        # Unique ID : optional
        if 'unique_id' in attrs:
            self.unique_id = attrs['unique_id']
        else:
            self.unique_id = None
        # Port : required
        self.port = attrs['port']
        # Hostname : required
        self.host_name = attrs['hostname']
        # IP address : optional
        if 'ip_address' in attrs:
            self.ip_address = attrs['ip_address']
        else:
            self.ip_address = None

    @staticmethod
    def _is_port_number(input):
        """
        Indicates if the input is a valid port number.
        :param input: Port number to check.
        :return: True if the input is a valid port number, false otherwise.
        """
        res = True
        try:
            port = int(input)
            if port < 1 or port > 65535:
                res = False
        except Exception, e:
            res = False
        return res



from threading import Condition
from base_test import BaseClientTest, atomize
from dxlclient import UuidGenerator, ServiceRegistrationInfo, Request, ErrorResponse
from dxlclient.test.test_service import TestService
from thread_executor import ThreadRunExecutor
import logging
import time
from nose.plugins.attrib import attr


class SyncRequestTroughputRunner(BaseClientTest):

    # The number of requests to send
    THREAD_COUNT = 100
    REQUEST_COUNT = 10

    # The maximum time for the test
    MAX_TIME = 10 * 60
    # The maximum time to wait between connections
    MAX_CONNECT_WAIT = 2 * 60
    # The number of times to try to connect to the broker
    MAX_CONNECT_RETRIES = 10

    connect_condition = Condition()
    response_count_condition = Condition()
    requests_start_time_condition = Condition()
    connect_time_condition = Condition()
    connect_retries_condition = Condition()
    cummulative_response_time_condition = Condition()

    _atomic_cummulative_response_time = 0
    _atomic_response_count = 0
    _atomic_connect_retries = 0
    _atomic_connect_count = 0
    _atomic_connect_time = 0
    _atomic_requests_start_time = 0

    requests_end_time = 0
    response_times = []

    @property
    @atomize(requests_start_time_condition)
    def requests_start_time(self):
        return self._atomic_requests_start_time

    @requests_start_time.setter
    @atomize(requests_start_time_condition)
    def requests_start_time(self, requests_start_time):
        self._atomic_requests_start_time = requests_start_time

    @property
    @atomize(connect_time_condition)
    def connect_time(self):
        return self._atomic_connect_time

    @connect_time.setter
    @atomize(connect_time_condition)
    def connect_time(self, connect_time):
        self._atomic_connect_time = connect_time

    @property
    def response_count(self):
        return self._atomic_response_count

    @response_count.setter
    def response_count(self, response_count):
        self._atomic_response_count = response_count

    @property
    @atomize(connect_retries_condition)
    def connect_retries(self):
        return self._atomic_connect_retries

    @connect_retries.setter
    @atomize(connect_retries_condition)
    def connect_retries(self, connect_retries):
        self._atomic_connect_retries = connect_retries

    @property
    @atomize(cummulative_response_time_condition)
    def cummulative_response_time(self):
        return self._atomic_cummulative_response_time

    @cummulative_response_time.setter
    @atomize(cummulative_response_time_condition)
    def cummulative_response_time(self, cummulative_response_time):
        self._atomic_cummulative_response_time = cummulative_response_time

    @property
    def atomic_connect_count(self):
        return self._atomic_connect_count

    @atomic_connect_count.setter
    def atomic_connect_count(self, connect_count):
        self._atomic_connect_count = connect_count

    @attr('load')
    def test_sync_request_troughput(self):
        self.execute_t(self.create_client)
        print self.get_execution_results()

    def get_execution_results(self):
        total_time = self.requests_end_time - self.requests_start_time
        output = "Connect time: " + str(self.connect_time) + "\n"
        output += "Connect retries: " + str(self.connect_retries) + "\n"
        output += "Request time: " + str(total_time) + "\n"
        output += "Total requests: " + str(self.THREAD_COUNT * self.REQUEST_COUNT) + "\n"
        output += "Requests/second: " + str((self.THREAD_COUNT * self.REQUEST_COUNT) / total_time) + "\n"
        output += "Average response time: " + str(self.cummulative_response_time /
                                                  (self.THREAD_COUNT * self.REQUEST_COUNT)) + "\n"

        mid = (self.THREAD_COUNT * self.REQUEST_COUNT) / 2
        self.response_times.sort()
        if (self.THREAD_COUNT * self.REQUEST_COUNT) % 2 == 0:
            median_response_time = self.response_times[mid] + self.response_times[mid-1] / 2
        else:
            median_response_time = self.response_times[mid]

        output += "Median response time: " + str(median_response_time)
        return output

    def execute_t(self, client_factory):

        with client_factory() as server_client:
            test_service = TestService(server_client, 1)
            connect_time_start = time.time()
            server_client.connect()
            topic = UuidGenerator.generate_id_as_string()
            reg_info = ServiceRegistrationInfo(server_client, "syncRequestThroughputRunnerService")
            reg_info.add_topic(topic, test_service)
            server_client.register_service_sync(reg_info, self.DEFAULT_TIMEOUT)

            #
            # Create a thread for each request. Wait for all of the clients to connect
            # to the broker before starting to calculate response related statistics.
            #

            executor = ThreadRunExecutor(self.THREAD_COUNT)

            def run():
                try:
                    with client_factory(max_retries=0) as client:
                        retries = self.MAX_CONNECT_RETRIES
                        connected = False
                        while not connected and retries > 0:
                            try:
                                client.connect()
                                connected = True
                            except Exception:
                                if retries > 0:
                                    retries -= 1
                                    self.connect_retries += 1
                        self.assertTrue(connected, "Unable to connect after retries")

                        # Waiting all clients have connected
                        with self.connect_condition:
                            self.atomic_connect_count += 1
                            self.connect_condition.notify_all()

                            while self.atomic_connect_count != self.THREAD_COUNT:
                                curr_count = self.atomic_connect_count
                                self.connect_condition.wait(self.MAX_CONNECT_WAIT)
                                if self.atomic_connect_count == curr_count:
                                    self.fail( "Timeout waiting for all threads to connect" )

                            # Once all clients have connected, reset timing information
                            if self.requests_start_time == 0:
                                self.requests_start_time = time.time()
                                self.connect_time = self.requests_start_time - connect_time_start

                        for i in range(0, self.REQUEST_COUNT):
                            req = Request(topic)
                            call_start_time = time.time()
                            response = client.sync_request(req, timeout=self.DEFAULT_TIMEOUT)
                            response_time = time.time() - call_start_time
                            self.assertNotIsInstance(response, ErrorResponse)

                            with self.response_count_condition:
                                self.response_count += 1
                                count = self.response_count
                                if (self.requests_end_time == 0) and \
                                        (count == (self.THREAD_COUNT * self.REQUEST_COUNT)):
                                    self.requests_end_time = time.time()

                            if count % 100 is 0:
                                print str(count) + ", " + str(time.time() - self.requests_start_time)

                            # Calulate and track response times
                            self.cummulative_response_time = self.cummulative_response_time + response_time
                            self.response_times.append(response_time)

                except Exception, e:
                    print e
                    logging.info(e.message)
                    raise e
            executor.execute(run)

            if self.THREAD_COUNT != self.response_count / self.REQUEST_COUNT:
                print "Failed! responseCount=" + str(self.response_count)
            self.assertEqual(self.THREAD_COUNT, self.response_count / self.REQUEST_COUNT)

            server_client.unregister_service_sync(reg_info, self.DEFAULT_TIMEOUT)



import logging
import os
import sys

from dxlclient.client import DxlClient
from dxlclient.client_config import DxlClientConfig
from dxlclient.message import Event

# Import common logging and configuration
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
from common import *

# Configure local logger
logger = logging.getLogger(__name__)

# Sample topic to publish DXL Events on
EVENT_TOPIC = "/isecg/sample/event"

# Sample Event Publisher
try:
    # Create DxlClientConfig from expected sample configuration file
    logger.info("Event Publisher - Load DXL config from: %s", CONFIG_FILE)
    config = DxlClientConfig.create_dxl_config_from_file(CONFIG_FILE)

    # Initialize DXL client using our configuration
    logger.info("Event Publisher - Creating DXL Client")
    with DxlClient(config) as client:

        # Connect to DXL Broker
        logger.info("Event Publisher - Connecting to Broker")
        client.connect()

        # Prompt user for input to publish DXL Events
        while True:
            print "   Enter 1 to publish a DXL Event"
            print "   Enter 9 to quit"
            input = raw_input("   Enter value: ")

            try:
                option = int(input)
            except:
                option = input

            # Option: DXL Event
            if option == 1:
                # Create the Event
                logger.info("Event Publisher - Creating Event for Topic %s", EVENT_TOPIC)
                event = Event(EVENT_TOPIC)

                # Encode string payload as UTF-8
                event.payload = "Sample Event Payload".encode()

                # Publish the Event to the DXL Fabric on the Topic
                logger.info("Event Publisher - Publishing Event to %s", EVENT_TOPIC)
                client.send_event(event)

            # Option: Exit the loop
            elif option == 9:
                break

            # Invalid input
            else:
                logger.info("Event Publisher - Invalid input: %s", option)
        # End Prompt Loop

except Exception as e:
    logger.info("Event Publisher - Exception: %s", e.message)
    exit(1)


# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################


class DxlException(Exception):
    """
    A general Data Exchange Layer (DXL) exception
    """


class MalformedBrokerUriException(DxlException):
    """
    An exception that is raised when a URL related to a DXL broker is malformed
    """


class WaitTimeoutException(DxlException):
    """
    Exception that is raised when a wait timeout is exceeded
    """


class BrokerListError(Exception):
    """
    Exception raised when a specified broker list is invalid
    """
    pass


def _raise_wrapped_exception(message, ex):
    """
     Wraps the specified exception as a {@link DxlException} with the specified message.
     If the incoming exception is already a {@link DxlException}, it is simply re-raised
     (no wrapping occurs).

    :param message: The message for the exception
    :param ex: The exception to wrap
    :return: None
    """
    if isinstance(ex, DxlException):
        raise ex
    else:
        raise DxlException(message, ex)



import logging
import time
from threading import Condition

from dxlclient import EventCallback, Event, UuidGenerator
from dxlclient.test.base_test import BaseClientTest, atomize
from dxlclient.test.thread_executor import ThreadRunExecutor
from nose.plugins.attrib import attr


class EventThroughputRunner(BaseClientTest):
    # The number of events to send
    THREAD_COUNT = 100
    EVENT_COUNT = 100

    # The maximum time for the test
    MAX_TIME = 10 * 60
    # The maximum time to wait between connections
    MAX_CONNECT_WAIT = 2 * 60
    # The number of times to try to connect to the broker
    MAX_CONNECT_RETRIES = 10

    connect_condition = Condition()
    connect_count_condition = Condition()
    event_count_condition = Condition()
    connect_time_condition = Condition()
    requests_start_time_condition = Condition()
    connect_retries_condition = Condition()

    _atomic_event_count = 0
    _atomic_connect_retries = 0
    _atomic_connect_count = 0
    _atomic_connect_time = 0
    _atomic_requests_start_time = 0

    connect_time_start = 0
    requests_end_time = 0
    response_times = []

    @property
    @atomize(requests_start_time_condition)
    def requests_start_time(self):
        return self._atomic_requests_start_time

    @requests_start_time.setter
    @atomize(requests_start_time_condition)
    def requests_start_time(self, requests_start_time):
        self._atomic_requests_start_time = requests_start_time

    @property
    @atomize(connect_time_condition)
    def connect_time(self):
        return self._atomic_connect_time

    @connect_time.setter
    @atomize(connect_time_condition)
    def connect_time(self, connect_time):
        self._atomic_connect_time = connect_time

    @property
    def event_count(self):
        return self._atomic_event_count

    @event_count.setter
    def event_count(self, event_count):
        self._atomic_event_count = event_count

    @property
    @atomize(connect_retries_condition)
    def connect_retries(self):
        return self._atomic_connect_retries

    @connect_retries.setter
    @atomize(connect_retries_condition)
    def connect_retries(self, connect_retries):
        self._atomic_connect_retries = connect_retries

    @property
    def atomic_connect_count(self):
        return self._atomic_connect_count

    @atomic_connect_count.setter
    def atomic_connect_count(self, connect_count):
        self._atomic_connect_count = connect_count

    @attr('event_test')
    def test_event_troughput(self):
        self.execute_t(self.create_client)
        # print self.get_statistics()

    def execute_t(self, client_factory):
        self.threads = 0
        with client_factory() as send_client:
            send_client.connect()
            event_topic = UuidGenerator.generate_id_as_string()
            executor = ThreadRunExecutor(self.THREAD_COUNT)

            def run():
                try:
                    with client_factory(max_retries=0) as client:
                        retries = self.MAX_CONNECT_RETRIES
                        connected = False
                        while not connected and retries > 0:
                            try:
                                self.connect_time_start = time.time()
                                client.connect()
                                connected = True
                            except Exception:
                                if retries > 0:
                                    retries -= 1
                                    self.connect_retries += 1

                        self.assertTrue(connected, "Unable to connect after retries")

                        def on_event(event):
                            with self.event_count_condition:
                                self.event_count += 1
                                current_count = self.event_count
                                self.event_count_condition.notify_all()

                                if current_count % 100 == 0:
                                    print client.config._client_id + " : " + str(current_count) + " : " + event.payload

                        # callback registration
                        callback = EventCallback()
                        callback.on_event = on_event
                        client.add_event_callback(event_topic, callback)

                        # Waiting all clients have connected
                        with self.connect_condition:
                            self.atomic_connect_count += 1
                            curr_count = self.atomic_connect_count

                            self.connect_condition.notify_all()
                            while self.atomic_connect_count != self.THREAD_COUNT:
                                self.connect_condition.wait(timeout=self.MAX_CONNECT_WAIT)
                                if curr_count == self.atomic_connect_count:
                                    self.fail("Timeout waiting for all threads to connect")

                            # Once all clients have connected, reset timing information
                            if self.requests_start_time == 0:
                                self.requests_start_time = time.time()
                                self.connect_time = self.requests_start_time - self.connect_time_start

                                for i in range(0, self.EVENT_COUNT):
                                    event = Event(event_topic)
                                    if i % 10 == 0:
                                        print "###send: " + str(i)
                                    event.payload = str(i)
                                    send_client.send_event(event)

                        with self.event_count_condition:
                            while self.event_count != self.EVENT_COUNT * self.THREAD_COUNT:
                                curr_count = self.event_count
                                self.event_count_condition.wait(timeout = self.MAX_CONNECT_WAIT)
                                if self.event_count == curr_count:
                                    self.fail("Timed out while receiving events")
                            self.event_count_condition.notify_all()
                            if self.requests_end_time == 0:
                                self.requests_end_time = time.time()

                except Exception, e:
                    logging.error(e.message)
                    raise e

            executor.execute(run)

            self.assertEquals(self.EVENT_COUNT * self.THREAD_COUNT, self.event_count)

            total_time = self.requests_end_time - self.requests_start_time
            print "Connect time: " + str(self.connect_time)
            print "Connect retries: " + str(self.connect_retries)
            print "Total events: " + str(self.EVENT_COUNT)
            print "Events/second: " + str(self.EVENT_COUNT / total_time)
            print "Total events received: " + str(self.EVENT_COUNT * self.THREAD_COUNT)
            print "Total events received/second: " + str((self.EVENT_COUNT * self.THREAD_COUNT) / total_time)
            print "Elapsed time: " + str(total_time)



from threading import Condition
from dxlclient import UuidGenerator, EventCallback, Event
from dxlclient.test.base_test import BaseClientTest, atomize
from nose.plugins.attrib import attr

@attr('system')
class EventTests(BaseClientTest):

    # The number of events to send
    EVENT_COUNT = 10000
    MAX_EVENT_WAIT = 2 * 60

    event_count = 0
    outstanding_events = []
    # Conditions
    event_condition = Condition()
    outstanding_events_condition = Condition()

    @atomize(outstanding_events_condition)
    def append_outstanding_event(self, message_id):
        self.outstanding_events.append(message_id)

    @atomize(outstanding_events_condition)
    def remove_outstanding_event(self, message_id):
        self.outstanding_events.remove(message_id)

    #
    # Tests the events-related methods of the DxlClient.
    #
    @attr('system')
    def test_execute_events(self):
        with self.create_client(max_retries=0) as client:
            try:
                client.connect()

                topic = UuidGenerator.generate_id_as_string()

                # Create and register an event callback. Ensure that all sent events are received
                # (via the outstanding events set). Also, track the number of total events received.
                def event_callback(event):
                    with self.event_condition:
                        # Increment count of responses received
                        self.event_count += 1
                        # Remove from outstanding events
                        self.remove_outstanding_event(event.message_id)
                        # Notify that a response has been received (are we done yet?)
                        self.event_condition.notify_all()

                ec = EventCallback()
                ec.on_event = event_callback

                client.add_event_callback(topic, ec)

                for i in range(0, self.EVENT_COUNT):
                    e = Event(topic)
                    self.append_outstanding_event(e.message_id)
                    client.send_event(e)

                with self.event_condition:
                    while self.event_count != self.EVENT_COUNT:                    
                        current_count = self.event_count
                        self.event_condition.wait(self.MAX_EVENT_WAIT)
                        if current_count == self.event_count:
                            self.fail("Event wait timeout.")

                self.assertEqual(0, len(self.outstanding_events))
                print "Events test: PASSED"

            except Exception, ex:
                print ex.message
                raise ex



import threading
import time
from threading import Condition

from dxlclient import UuidGenerator, ServiceRegistrationInfo, Request, ErrorResponse, EventCallback, Event
from dxlclient.test.base_test import BaseClientTest
from dxlclient.test.test_service import TestService
from nose.plugins.attrib import attr

@attr('system')
class DxlClientTest (BaseClientTest):

    #
    # Tests the connect and disconnect methods of the DxlClient
    #
    @attr('system')
    def test_dxl_client_connection(self):
        self.execute_con_disc_test()

    def execute_con_disc_test(self):
        with self.create_client(max_retries=0) as client:
            try:
                client.connect()
                client.disconnect()
            except Exception, e:
                print e.message

    #
    # Tests the subscribe and unsubscribe methods of the DxlClient
    #
    @attr('system')
    def test_dxlclient_subscription(self):
        self.execute_subs__unsubs_test()

    def execute_subs__unsubs_test(self):
        with self.create_client(max_retries=0) as client:
            try:
                client.connect()
                topic = UuidGenerator.generate_id_as_string()
                client.subscribe(topic)
                time.sleep(0.5)
                client.unsubscribe(topic)
                client.disconnect()
            except Exception, e:
                print e.message

    #
    # Test to ensure that ErrorResponse messages can be successfully delivered
    # from a service to a client.
    #
    @attr('system')
    def test_error_message(self):
        with self.create_client() as client:
            test_service = TestService(client, 1)
            client.connect()

            error_code = 9090
            error_message = "My error message"

            topic = UuidGenerator.generate_id_as_string()

            #
            # Create a test service that returns error messages
            #

            reg_info = ServiceRegistrationInfo(client, "testErrorMessageService")
            reg_info.add_topic(topic, test_service)
            client.register_service_sync(reg_info, self.DEFAULT_TIMEOUT)

            test_service.return_error = True
            test_service.error_code = error_code
            test_service.error_message = error_message
            client.add_request_callback(topic, test_service)

            # Send a request and ensure the response is an error message
            response = client.sync_request(Request(topic))
            self.assertIsInstance(response, ErrorResponse, msg="Response is not an ErrorResponse")
            self.assertEquals(error_code, response.error_code)
            self.assertEquals(error_message, response.error_message)

    #
    # Tests threading of incoming requests
    #
    @attr('system')
    def test_incoming_message_threading(self):
        thread_count = 10
        thread_name = set()

        event_topic = UuidGenerator.generate_id_as_string()
        with self.create_client(incoming_message_thread_pool_size=thread_count) as client:
            client.connect()
            event_callback = EventCallback()

            def on_event(event):
                thread_name.add(threading.current_thread())

            event_callback.on_event = on_event
            client.add_event_callback(event_topic, event_callback)

            for i in range(0, 1000):
                evt = Event(event_topic)
                client.send_event(evt)

            time.sleep(30)

            self.assertEquals(thread_count, len(thread_name))




# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################
import os

################################################################################
#
# Global Configuration
#
################################################################################

if os.name.lower() == "posix":
    PATH_CACHE = "/var/McAfee/{0}".format("dxlclient")
else:
    PATH_CACHE = "./.{0}".format("dxlclient")

PATH_CONFIG = PATH_CACHE + '/conf'
PATH_KEYSTORE = PATH_CACHE + '/keystore'
PATH_LOGS = PATH_CACHE + '/logs'

FILE_CA_BUNDLE = "cabundle.pem"

FILE_CERT_PFX = "dxlcert.p12"
FILE_CERT_PEM = "dxlcert.pem"
FILE_DXL_PRIVATE_KEY = "dxlprivatekey.pem"

FILE_CONFIG = PATH_CONFIG + '/config'


def get_cache_dir():
    """
    Returns current cache folder.

    :returns: {@code string}: Cache folder.
    """
    return PATH_CACHE


def get_config_dir():
    """
    Returns current configuration folder.

    :returns: {@code string}: Configuration folder.
    """
    return PATH_CONFIG


def get_keystore_dir():
    """
    Returns current keystore folder.

    :returns: {@code string}: Keystore folder.
    """
    return PATH_KEYSTORE


def get_logs_dir():
    """
    Returns current logs folder.

    :returns: {@code string}: Logs folder.
    """
    return PATH_LOGS


def get_ca_bundle_pem():
    """
    Returns current CA Bundle filename.

    :returns: {@code string}: CA Bundle filename.
    """
    return os.path.join(PATH_KEYSTORE, FILE_CA_BUNDLE)


def get_cert_file_pfx():
    """
    Returns current PFX certificate filename.

    :returns: {@code string}: PFX certificate filename.
    """
    return os.path.join(PATH_KEYSTORE, FILE_CERT_PFX)


def get_cert_file_pem():  # pylint: disable=invalid-name
    """
    Returns current PEM certificate filename.

    :returns: {@code string}: PEM certificate filename.
    """
    return os.path.join(PATH_KEYSTORE, FILE_CERT_PEM)

def get_dxl_private_key():  # pylint: disable=invalid-name
    """
    Returns current PEM certificate filename.

    :returns: {@code string}: PEM certificate filename.
    """
    return os.path.join(PATH_KEYSTORE, FILE_DXL_PRIVATE_KEY)

def get_dxl_config_file():
    """
    Returns default dxl config file

    :return:
    """
    return FILE_CONFIG



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

from dxlclient import _BaseObject


class MessageCallback(_BaseObject):  # pylint: disable=too-few-public-methods
    """
    Base class for the different callbacks
    """
    def __init__(self):
        super(MessageCallback, self).__init__()

    def __del__(self):
        super(MessageCallback, self).__del__()


class EventCallback(MessageCallback):  # pylint: disable=too-few-public-methods
    """
    Concrete instances of this interface are used to receive :class:`dxlclient.message.Event` messages.

    To receive events, a concrete instance of this callback must be created and registered with a
    :class:`dxlclient.client.DxlClient` instance via the :func:`dxlclient.client.DxlClient.add_event_callback`
    method.

    The following is a simple example of using an event callback:

    .. code-block:: python

        from dxlclient.callbacks import EventCallback

        class MyEventCallback(EventCallback):
            def on_event(self, event):
                print "Received event! " + event.source_client_id

        dxl_client.add_event_callback("/testeventtopic", MyEventCallback())

    **NOTE:** By default when registering an event callback the client will automatically subscribe
    (:func:`dxlclient.client.DxlClient.subscribe`) to the topic.

    The following demonstrates a client that is sending an event message that would be received by the
    callback above.

    .. code-block:: python

        from dxlclient.message import Event

        # Create the event message
        evt = Event("/testeventtopic")

        # Populate the event payload
        evt.payload = "testing".encode()

        # Send the event
        dxl_client.send_event(evt)
    """

def on_event(self, event):
    """
    Invoked when an :class:`dxlclient.message.Event` has been received.

    :param event: The :class:`dxlclient.message.Event` message that was received
    """
    raise NotImplementedError("Must be implemented in a child class.")


class RequestCallback(MessageCallback):  # pylint: disable=too-few-public-methods
    """
    Concrete instances of this interface are used to receive :class:`dxlclient.message.Request` messages.

    Request callbacks are typically used when implementing a "service".

    See :class:`dxlclient.service.ServiceRegistrationInfo` for more information on how to register a
    service.
    """
    def on_request(self, request):
        """
        Invoked when an :class:`dxlclient.message.Request` has been received.

        :param request: The :class:`dxlclient.message.Request` message that was received
        """
        raise NotImplementedError("Must be implemented in a child class.")


class ResponseCallback(MessageCallback): # pylint: disable=too-few-public-methods
    """
    Concrete instances of this interface are used to receive :class:`dxlclient.message.Response` messages.

    Response callbacks are typically used when invoking a service asynchronously.

    The following is a simple example of using a response callback with an asynchronous service invocation:

    .. code-block:: python

        from dxlclient.message import Request
        from dxlclient.callbacks import ResponseCallback

        class MyResponseCallback(ResponseCallback):
            def on_response(self, response):
                print "Received response! " + response.service_id

        request = Request("/testservice/testrequesttopic")
        dxl_client.async_request(request, MyResponseCallback())

    """

    def on_response(self, response):
        """
        Invoked when an :class:`dxlclient.message.Response` has been received.

        :param request: The :class:`dxlclient.message.Response` message that was received
        """
        raise NotImplementedError("Must be implemented in a child class.")



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

import threading
import types

from dxlclient import _BaseObject
from dxlclient.callbacks import MessageCallback, RequestCallback, ResponseCallback, EventCallback
from dxlclient._dxl_utils import WildcardCallback, DxlUtils


def _has_wildcard(channel_name):
    """
    Return whether the channel has a wildcard

    :param channel_name: The channel name
    :return: Whether the channel has a wildcard
    """
    if not  isinstance(channel_name, str):
        raise ValueError("Channel name should be str class")
    return channel_name and channel_name[-1] == '#'


class _CallbackManager(_BaseObject):
    def __init__(self):
        super(_CallbackManager, self).__init__()

        # Read write lock for handling registration and firing concurrency
        self.lock = threading.RLock()
        # Map containing registered listeners by channel name (empty indicates all channels)
        self.callbacks_by_channel = {}
        # Is wildcarding enabled
        self.wildcarding_enabled = False

    def validate_callback(self, callback):  # pylint: disable=no-self-use
        """
        Validates if `callback` is a valid MessageCallback.

        :param callback: Callback to validate.
        """
        if callback is None:
            raise ValueError("Missing callback argument")
        # Check if the provided eventCallback is a class
        if isinstance(callback, (type, types.ClassType)):
            if not issubclass(callback, MessageCallback):
                raise ValueError("Type mismatch on callback argument")
        # Not a class, but an instance
        else:
            if not issubclass(callback.__class__, MessageCallback):
                raise ValueError("Type mismatch on callback argument")

    def add_callback(self, channel="", callback=None):
        """
        Adds the specified callback. The callback will receive messages that were received
        via the specified channel

        :param channel: Limits messages sent to the callback that were received via this channel
        :param callback: The callback to add
        :return: True if the callback was added successfully; False otherwise
        """
        if channel is None:
            raise ValueError("Missing channel argument")

        self.validate_callback(callback)

        rc = False  # pylint: disable=invalid-name
        with self.lock:
            if _has_wildcard(channel):
                self.wildcarding_enabled = True
            callbacks = self.callbacks_by_channel.get(channel)
            if callbacks is None:
                callbacks = []
            if not callback in callbacks:
                callbacks.append(callback)
                self.callbacks_by_channel[channel] = callbacks
                rc = True  # pylint: disable=invalid-name
        return rc

    def remove_callback(self, channel="", callback=None):
        """
        Removes the callback that was registered for the specified channel.

        :param channel: The channel name
        :param callback: The callback to remove
        :return: True if the callback was removed successfully; False otherwise
        """
        if channel is None:
            raise ValueError("Missing channel argument")

        self.validate_callback(callback)

        rc = False  # pylint: disable=invalid-name
        with self.lock:
            callbacks = self.callbacks_by_channel.get(channel)
            if callbacks is not None:
                callbacks.remove(callback)
                if len(callbacks) == 0:
                    del self.callbacks_by_channel[channel]
                else:
                    self.callbacks_by_channel[channel] = callbacks
                rc = True  # pylint: disable=invalid-name

            #Determine if any wildcard exist
            if self.wildcarding_enabled:
                self.wildcarding_enabled = False
                for current_channel_name in self.callbacks_by_channel.keys():
                    if _has_wildcard(current_channel_name):
                        self.wildcarding_enabled = True
                        break
        return rc

    def fire_message(self, message):
        """
        Fires the specified message to the appropriate (taking into consideration
        channel) registered listeners.

        :param message: The message to fire
        :return: None
        """
        with self.lock:
            # Fire for global listeners (channel="")
            self._fire_message(self.callbacks_by_channel.get(""), message)
            # Fire for channel listeners
            self._fire_message(self.callbacks_by_channel.get(message.destination_topic), message)
            #Fire for all wildcarded channels
            # If wildcarding is enabled the message will be fired to each of the message's destination
            # wildcards, if such wildcard exists.
            if self.wildcarding_enabled:

                def on_next_wildcard(wildcard):
                    #if wildcarded channel does not exist no message is fired
                    self._fire_message(self.callbacks_by_channel.get(wildcard), message)

                wildcard_callback = WildcardCallback()
                wildcard_callback.on_next_wildcard = on_next_wildcard

                #Iterate over all channel wildcards sending messages via callback if
                DxlUtils.iterate_wildcards(wildcard_callback, message.destination_topic)

    def _fire_message(self, callbacks, message):
        """
        Fires the message to the specified set of callbacks.

        :param callbacks: The callbacks to fire the message to
        :param message: The message to fire
        :return:
        """
        if callbacks is None or len(callbacks) == 0:
            return

        for callback in callbacks:
            self.handle_fire(callback, message)

    def handle_fire(self, callback, message):
        """
        Method that is to be overridden by derived classes to fire the message to the
        specified callback.

        :param callback: The target of the message
        :param message: The message
        """
        pass


class _RequestCallbackManager(_CallbackManager):
    """
    Manager for {@link RequestCallback} message callbacks.
    """

    def validate_callback(self, callback):
        """
        Validates if `callback` is a valid RequestCallback.

        :param callback: Callback to validate.
        """
        super(_RequestCallbackManager, self).validate_callback(callback)
        # Check if the provided callback is a class
        if isinstance(callback, (type, types.ClassType)):
            if not issubclass(callback, RequestCallback):
                raise ValueError("Type mismatch on callback argument")
        # Not a class, but an instance
        else:
            if not issubclass(callback.__class__, RequestCallback):
                raise ValueError("Type mismatch on callback argument")

    def handle_fire(self, request_callback, request):
        """
        Runs `request_callback` for `request`.

        :param request_callback: {@link dxlclient.callbacks.RequestCallback} object that will handle the event.
        :param request: {@link dxlclient.request.Request} object.
        """
        # Check if the provided eventCallback is a class
        if isinstance(request_callback, (type, types.ClassType)):
            callback = request_callback()
            callback.on_request(request)
        # Not a class, but an instance
        else:
            request_callback.on_request(request)


class _ResponseCallbackManager(_CallbackManager):
    """
    Manager for {@link ResponseCallback} message callbacks.
    """

    def validate_callback(self, callback):
        """
        Validates if `callback` is a valid ResponseCallback.

        :param callback: Callback to validate.
        """
        super(_ResponseCallbackManager, self).validate_callback(callback)
        # Check if the provided callback is a class
        if isinstance(callback, (type, types.ClassType)):
            if not issubclass(callback, ResponseCallback):
                raise ValueError("Type mismatch on callback argument")
        # Not a class, but an instance
        else:
            if not issubclass(callback.__class__, ResponseCallback):
                raise ValueError("Type mismatch on callback argument")

    def handle_fire(self, response_callback, response):
        """
        Runs `response_callback` for `response`.

        :param response_callback: {@link dxlclient.callbacks.ResponseCallback} object that will handle the event.
        :param response: {@link dxlclient.response.Response} object.
        """
        # Check if the provided eventCallback is a class
        if isinstance(response_callback, (type, types.ClassType)):
            callback = response_callback()
            callback.on_response(response)
        # Not a class, but an instance
        else:
            response_callback.on_response(response)


class _EventCallbackManager(_CallbackManager):
    """
    Manager for {@link EventCallback} message callbacks.
    """

    def validate_callback(self, callback):
        """
        Validates if `callback` is a valid EventCallback.

        :param callback: Callback to validate.
        """
        super(_EventCallbackManager, self).validate_callback(callback)
        # Check if the provided callback is a class
        if isinstance(callback, (type, types.ClassType)):
            if not issubclass(callback, EventCallback):
                raise ValueError("Type mismatch on callback argument")
        # Not a class, but an instance
        else:
            if not issubclass(callback.__class__, EventCallback):
                raise ValueError("Type mismatch on callback argument")

    def handle_fire(self, event_callback, event):
        """
        Runs `event_callback` for `event`.

        :param event_callback: {@link dxlclient.callbacks.EventCallback} object that will handle the event.
        :param event: {@link dxlclient.event.Event} object.
        """
        # Check if the provided eventCallback is a class
        if isinstance(event_callback, (type, types.ClassType)):
            callback = event_callback()
            callback.on_event(event)
        # Not a class, but an instance
        else:
            event_callback.on_event(event)



# This sample demonstrates wrapping an existing service and exposing it on the
# DXL fabric.
#
# In this particular case, the "openweathermap.org" current weather data is
# exposed as a DXL service. This service wrapper delegates to the
# OpenWeatherMap REST API.

# The API key for invoking the weather service
API_KEY = "<obtain key from http://openweathermap.org/appid>"

# The name of the OpenWeatherMap service
SERVICE_NAME = "/openweathermap/service/openweathermap"

# The "current weather" topic
SERVICE_CURRENT_WEATHER_TOPIC = SERVICE_NAME + "/current"



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

import threading
import time
import logging

from dxlclient.exceptions import _raise_wrapped_exception, WaitTimeoutException
from dxlclient.callbacks import ResponseCallback

logger = logging.getLogger(__name__)


# pylint: disable=too-many-instance-attributes
class RequestManager(ResponseCallback):
    """
    Manager that tracks outstanding requests and notifies the appropriate parties
    (invoking a response callback, notifying a waiting object, etc.) when a corresponding
    response is received.

    This purpose of this object is to collaborate with an {@link DxlClient}
    instance.
    """
    # TODO: Add TTL to removed callbacks if response is never received.
    # TODO: Remove current message identifiers if response is never received.

    def __init__(self, client):
        """Constructor"""
        super(RequestManager, self).__init__()

        # Map containing {@link ResponseCallback} instances associated with the
        # identifier of the request message that they are waiting for a response
        # to. This map is used for asynchronous requests.
        self.callback_map = {}
        # Lock for request threads waiting for a response (for synchronous request)
        self.sync_wait_message_lock = threading.RLock()
        # The condition associated with the request threads waiting for a response
        # (for synchronous request).
        self.sync_wait_message_condition = threading.Condition(self.sync_wait_message_lock)
        # Set of identifiers of request messages that are waiting for a response
        # (for synchronous request).
        self.sync_wait_message_ids = set()
        # Messages that have been received mapped by the corresponding request
        # message identifier (for synchronous request).
        self.sync_wait_message_responses = {}
        # Lock for the current request message identifiers (requests that are in process)
        self.current_request_message_lock = threading.RLock()
        # Current request message identifiers (requests that are in process)
        self.current_request_message_ids = set()
        # The client that the request manager is associated with
        self.client = client
        # Register self as ResponseCallback for all channels
        self.client.add_response_callback("", self)

    def destroy(self):
        """Destroys the service manager (releases resources)"""
        self.client = None

    def add_current_request(self, message_id):
        """
        Adds the specified message identifier to the "current" set of requests
        (requests that are in process).
        :param message_id: The request message identifier
        :return: None
        """
        with self.current_request_message_lock:
            if not message_id in self.current_request_message_ids:
                self.current_request_message_ids.add(message_id)

    def remove_current_request(self, message_id):
        """
        Removes the specified message identifier from the "current" set of requests
        (requests that are in process).
        :param message_id: The request message identifier
        :return: None
        """
        with self.current_request_message_lock:
            if message_id in self.current_request_message_ids:
                self.current_request_message_ids.remove(message_id)

    def get_current_request_queue_size(self):
        """
        Returns the size of the current request queue
        :return: The size of the current request queue
        """
        with self.current_request_message_lock:
            return len(self.current_request_message_ids)

    def sync_request(self, request, wait):
        """
        Performs a synchronous request with the default timeout via the DXL fabric
        :param request: The request
        :param wait_ms: The maximum time to wait for the request
        :return: The Response object
        """
        response = None
        self.register_wait_for_response(request)
        try:
            try:
                # Add to set of current requests
                self.add_current_request(request.message_id)
                try:
                    self.client._send_request(request)
                    response = self.wait_for_response(request, wait)
                except Exception as ex:
                    _raise_wrapped_exception("Error during synchronous request", ex)
            finally:
                # Remove from set of current requests
                self.remove_current_request(request.message_id)
        finally:
            self.unregister_wait_for_response(request)

        return response

    def async_request(self, request, response_callback):
        """
        Performs an asynchronous request via the DXL fabric
        :param request: The request
        :param response_callback: The callback to be invoked when the response is received
        :return: None
        """
        destination = request.destination_topic
        if not response_callback is None:
            self.register_async_callback(request, response_callback)
        try:
            # Add to set of current requests
            self.add_current_request(request.message_id)
            self.client._send_request(request)
        except Exception as ex:
            try:
                if not response_callback is None:
                    self.unregister_async_callback(destination)
            finally:
                self.remove_current_request(request.message_id)
            _raise_wrapped_exception("Error during asynchronous request", ex)

    def register_wait_for_response(self, request):
        """
        Indicates to the request manager that you are about to wait for the specified
        request (synchronous response). The registration has to occur prior to actually
        sending the request message to account for the possibility of the response being
        received immediately.
        :param request: The request that is about to be waited for.
        :return: None
        """
        with self.sync_wait_message_lock:
            self.sync_wait_message_ids.add(request.message_id)


    def unregister_wait_for_response(self, request):
        """
        Indicates to the request manager that you no longer want to wait for the specified
        request (synchronous response). This must be invoked when an error occurs while
        waiting for the response, or the response was received.
        :param request: The request that should no longer be waited for
        :return: None
        """
        with self.sync_wait_message_lock:
            if request.message_id in self.sync_wait_message_ids:
                self.sync_wait_message_ids.remove(request.message_id)
            if request.message_id in self.sync_wait_message_responses:
                del self.sync_wait_message_responses[request.message_id]

    def register_async_callback(self, request, response_callback):
        """
        Registers an asynchronous callback for the specified request
        :param request: The request
        :param response_callback: The callback to invoke when the response to request is received.
        :return: None
        """
        self.callback_map[request.message_id] = response_callback

    def unregister_async_callback(self, message_id):
        """
        Removes an asynchronous callback for the specified request
        :param message_id: The identifier for the request
        :return: The response callback or None, if not available
        """
        if message_id in self.callback_map:
            response_callback = self.callback_map[message_id]
            del self.callback_map[message_id]
        else:
            response_callback = None
        return response_callback

    def _get_async_callback_count(self):
        """
        Returns the count of async callbacks that are waiting for a response
        :return: The count of async callbacks that are waiting for a response
        """
        return len(self.callback_map)

    def wait_for_response(self, request, wait):
        """
        Waits for a response to the specified request up to the specified wait time
        in milliseconds.
        :param request: The request for which to wait for the response
        :param wait: The maximum time to wait for the request
        :return: The Response object
        """
        message_id = request.message_id

        with self.sync_wait_message_lock:
            wait_seconds = wait
            wait_start = time.time()

            while not message_id in self.sync_wait_message_responses:
                self.sync_wait_message_condition.wait(wait_seconds)
                if (time.time() - wait_start) >= wait_seconds:
                    raise WaitTimeoutException("Timeout waiting for response to message: " + message_id)

            response = self.sync_wait_message_responses[message_id]
            del self.sync_wait_message_responses[message_id]

        return response

    def on_response(self, response):
        """
        Invoked when an Response has been received.
        """
        request_message_id = response.request_message_id
        try:
            # Check for synchronous waits
            with self.sync_wait_message_lock:
                if request_message_id in self.sync_wait_message_ids:
                    self.sync_wait_message_ids.remove(request_message_id)
                    self.sync_wait_message_responses[request_message_id] = response
                    self.sync_wait_message_condition.notifyAll()

            # Check for asynchronous callbacks
            callback = self.unregister_async_callback(request_message_id)
            if not callback is None:
                callback.on_response(response)
        finally:
            # Remove from set of current requests
            self.remove_current_request(request_message_id)



import time
from dxlclient import ServiceRegistrationInfo, UuidGenerator
from dxlclient import RequestCallback, Response, Message, ResponseCallback, Request
from dxlclient.test.base_test import BaseClientTest
from threading import Condition
from nose.plugins.attrib import attr


#
# Slams a service with a flood of asynchronous tests. The PAHO library by default will
# deadlock when it is waiting to complete a publish and at the same time receives an
# incoming message.
#
# This test ensures that the changes we made to the PAHO library now work in this particular
# scenario.
#
@attr('system')
class AsyncFloodTest(BaseClientTest):
    # The count of requests to send
    REQUEST_COUNT = 1000

    # Amount of time to wait for the test to succeed
    WAIT_TIME = 90

    # The service registration information
    m_info = None

    resp_condition = Condition()
    response_count = 0
    error_count = 0

    @attr('system')
    def test_async_flood(self):

        channel = UuidGenerator.generate_id_as_string()

        with self.create_client() as client:

            self.m_info = ServiceRegistrationInfo(client, channel)
            client.connect()
            client.subscribe(channel)

            def my_request_callback(rq):
                # print "request: " + str(rq.version)
                try:
                    time.sleep(0.05)
                    resp = Response(rq)
                    resp.payload = rq.payload
                    client.send_response(resp)
                except Exception, e:
                    print e.message

            req_callback = RequestCallback()
            req_callback.on_request = my_request_callback

            self.m_info.add_topic(channel, req_callback)
            client.register_service_sync(self.m_info, 10)

            with self.create_client() as client2:
                client2.connect()

                def my_response_callback(response):
                    # print "response"
                    if response.message_type == Message.MESSAGE_TYPE_ERROR:
                        print "Received error response: " + response._error_response
                        with self.resp_condition:
                            self.error_count += 1
                            self.resp_condition.notify_all()
                    else:
                        with self.resp_condition:
                            if self.response_count % 10 == 0:
                                print "Received request " + str(self.response_count)
                            self.response_count += 1
                            self.resp_condition.notify_all()

                callback = ResponseCallback()
                callback.on_response = my_response_callback

                client2.add_response_callback("", callback)

                for i in range(0, self.REQUEST_COUNT):
                    if i % 100 == 0:
                        print "Sent: " + str(i)

                    request = Request(channel)
                    pl = str(i)
                    request.payload = pl
                    client2.async_request(request)

                    if self.error_count > 0:
                        break

                # Wait for all responses, an error to occur, or we timeout
                start_time = time.time()
                with self.resp_condition:
                    while (self.response_count != self.REQUEST_COUNT) and (self.error_count == 0) and (time.time() - start_time < self.WAIT_TIME):
                        self.resp_condition.wait(5)

                if self.error_count != 0:
                    exc = Exception()
                    exc.message = "Received an error response!"
                    raise exc

                self.assertEquals(self.REQUEST_COUNT, self.response_count, "Did not receive all messages!")



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

import traceback
from Queue import Queue
from threading import Thread
import logging

from dxlclient import _BaseObject, _ObjectTracker
from dxlclient._uuid_generator import UuidGenerator

logger = logging.getLogger(__name__)


class ThreadPoolWorker(Thread):
    """
    Thread executing tasks from a given tasks queue.
    """

    def __init__(self, tasks, thread_prefix):
        """
        Constructs a ThreadPoolWorker.
        """
        Thread.__init__(self)

        _ObjectTracker.get_instance().obj_constructed(self)

        self.tasks = tasks
        self.daemon = True
        self.name = thread_prefix + "-" + UuidGenerator.generate_id_as_string()
        self.start()

    def __del__(self):
        _ObjectTracker.get_instance().obj_destructed(self)

    def run(self):
        """
        Runs the worker.
        """
        while True:
            func, args, kargs = self.tasks.get()
            try:
                if func is None:
                    # Exit the thread
                    return
                else:
                    func(*args, **kargs)  # pylint: disable=star-args
            except Exception as ex:  # pylint: disable=broad-except
                logger.error("Error in worker thread: %s", ex.message)
                logger.debug(traceback.format_exc())
            del func
            self.tasks.task_done()


class ThreadPool(_BaseObject):
    """
    Pool of threads consuming tasks from a queue.
    """

    def __init__(self, queue_size, num_threads, thread_prefix):
        """
        Creates a ThreadPool.
        """
        super(ThreadPool, self).__init__()
        self._tasks = Queue(queue_size)
        self._threads = []
        for _ in range(num_threads):
            t = ThreadPoolWorker(self._tasks, thread_prefix)
            self._threads.append(t)

    def __del__(self):
        """destructor"""
        super(ThreadPool, self).__del__()

    def add_task(self, func, *args, **kargs):
        """Add a task to the queue"""
        self._tasks.put((func, args, kargs))

    def wait_completion(self):
        """Wait for completion of all the tasks in the queue"""
        self._tasks.join()

    def shutdown(self):
        """Shuts down the thread pool"""
        logger.info("Shutting down thread pool...")
        self.wait_completion()

        # Add task to stop the thread
        for _ in self._threads:
            self.add_task(None)

        # Wait for threads to exit
        for t in self._threads:
            t.join()




import logging
import os
import sys

from dxlclient.callbacks import RequestCallback
from dxlclient.client import DxlClient
from dxlclient.client_config import DxlClientConfig
from dxlclient.message import Response
from dxlclient.service import ServiceRegistrationInfo

# Import common logging and configuration
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
from common import *

# Configure local logger
logger = logging.getLogger(__name__)

# Sample topic to fire Requests on
SERVICE_TOPIC = "/isecg/sample/service"

# Sample Service Provider
try:
    # Create DxlClientConfig from expected sample configuration file
    logger.info("Service Provider - Load DXL config from: %s", CONFIG_FILE)
    config = DxlClientConfig.create_dxl_config_from_file(CONFIG_FILE)

    # Initialize DXL client using our configuration
    logger.info("Service Provider - Creating DXL Client")
    with DxlClient(config) as client:

        # Connect to DXL Broker
        logger.info("Service Provider - Connecting to Broker")
        client.connect()

        # Response callback class to handle DXL Responses to our Asynchronous Requests
        class MyRequestCallback(RequestCallback):
            def on_request(self, request):
                # Extract information from Response payload, in this sample we expect it is UTF-8 encoded
                logger.info("Service Provider - Request received:\n   Topic: %s\n   Request ID: %s\n   Payload: %s",
                            request.destination_topic,
                            request.message_id,
                            request.payload.decode())

                # Create the Response message
                logger.info("Service Provider - Creating Response for Request ID %s on %s",
                            request.message_id, request.destination_topic)
                response = Response(request)

                # Encode string payload as UTF-8
                response.payload = "Sample Response Payload".encode()

                # Send the Response back
                logger.info("Service Provider - Sending Response to Request ID: %s on %s",
                            response.request_message_id, request.destination_topic)
                client.send_response(response)

        # Create DXL Service Registration object
        service_registration_info = ServiceRegistrationInfo(client, "/mycompany/myservice")

        # Add a topic for the service to respond to
        service_registration_info.add_topic(SERVICE_TOPIC, MyRequestCallback())

        # Register the service with the DXL fabric (with a wait up to 10 seconds for registration to complete)
        logger.info("Registering service.")
        client.register_service_sync(service_registration_info, 10)

        # Wait for DXL Requests
        while True:
            print "   Enter 9 to quit"
            input = raw_input("   Enter value: ")

            try:
                option = int(input)
            except:
                option = input

            # Option: Exit the loop
            if option == 9:
                break

            # Invalid input
            else:
                logger.info("Service Provider - Invalid input: %s", option)

except Exception as e:
    logger.info("Service Provider - Exception: %s", e.message)
    exit(1)


import time
from dxlclient import UuidGenerator, Request, EventCallback, Event
from dxlclient.test.base_test import BaseClientTest
from nose.plugins.attrib import attr

@attr('system')
class TestSyncDuringCallback(BaseClientTest):

    SLEEP_TIME = 5
    exceptions = []

    # Test to ensure that synchronous requests can't be made on
    # the incoming message thread
    def test_execute_sync_during_callback(self):

        event_topic = UuidGenerator.generate_id_as_string()
        req_topic = UuidGenerator.generate_id_as_string()

        with self.create_client() as client:
            client.connect()

            # callback
            def event_callback(event):
                try:
                    req = Request(destination_topic=req_topic)
                    client.sync_request(req)
                except Exception, e:
                    self.exceptions.append(e)
                    raise e

            ec = EventCallback()
            ec.on_event = event_callback

            client.add_event_callback(event_topic, ec)

            time.sleep(self.SLEEP_TIME)  # Check the time

            evt = Event(destination_topic=event_topic)
            client.send_event(evt)

            time.sleep(self.SLEEP_TIME)

            self.assertTrue(self.exceptions[0] is not None)
            self.assertTrue("different thread" in self.exceptions[0].message)



from concurrent.futures import ThreadPoolExecutor


class ThreadRunExecutor(object):

    futures = []

    def __init__(self, run_count):
        self.executor = ThreadPoolExecutor(max_workers=run_count)
        self.run_count = run_count

    def execute(self, command):
        with self.executor as executor:
            for i in range(0, self.run_count):
                self.futures.append(executor.submit(command))



# This sample demonstrates how to register a DXL service to receive Request
# messages and send Response messages back to an invoking client.

import logging
import os
import sys

from dxlclient.callbacks import RequestCallback
from dxlclient.client import DxlClient
from dxlclient.client_config import DxlClientConfig
from dxlclient.message import Message, Request, Response
from dxlclient.service import ServiceRegistrationInfo

# Import common logging and configuration
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
from common import *

# Configure local logger
logging.getLogger().setLevel(logging.ERROR)
logger = logging.getLogger(__name__)

# The topic for the service to respond to
SERVICE_TOPIC = "/isecg/sample/basicservice"

# Create DXL configuration from file
config = DxlClientConfig.create_dxl_config_from_file(CONFIG_FILE)

# Create the client
with DxlClient(config) as client:

    # Connect to the fabric
    client.connect()

    #
    # Register the service
    #

    # Create incoming request callback
    class MyRequestCallback(RequestCallback):
        def on_request(self, request):
            # Extract information from request
            print "Service received request payload: " + request.payload.decode()
            # Create the response message
            res = Response(request)
            # Populate the response payload
            res.payload = "pong".encode()
            # Send the response
            client.send_response(res)

    # Create service registration object
    info = ServiceRegistrationInfo(client, "myService")

    # Add a topic for the service to respond to
    info.add_topic(SERVICE_TOPIC, MyRequestCallback())

    # Register the service with the fabric (wait up to 10 seconds for registration to complete)
    client.register_service_sync(info, 10)

    #
    # Invoke the service (send a request)
    #

    # Create the request message
    req = Request(SERVICE_TOPIC)

    # Populate the request payload
    req.payload = "ping".encode()

    # Send the request and wait for a response (synchronous)
    res = client.sync_request(req)

    # Extract information from the response (if an error did not occur)
    if res.message_type != Message.MESSAGE_TYPE_ERROR:
        print "Client received response payload: " + res.payload.decode()


from threading import Condition
from dxlclient import UuidGenerator, ServiceRegistrationInfo, RequestCallback, Request
from dxlclient.test.base_test import BaseClientTest
from dxlclient.test.test_service import TestService
from nose.plugins.attrib import attr
from dxlclient._vendor.msgpack.fallback import Unpacker
from dxlclient._vendor.msgpack.fallback import Packer
import StringIO

@attr('system')
class MessagePayloadTest(BaseClientTest):

    MAX_WAIT = 1 * 60
    # A test string to send
    TEST_STRING = "SslUtils"
    # A test byte to send
    TEST_BYTE = 1
    # A test integer
    TEST_INT = 123456

    received_request = False
    request_complete_condition = Condition()

    # Tests whether payloads can be successfully delivered from a client to the server.
    # Payloads are simply bytes of data that are used to provide application-specific
    # information.
    @attr('system')
    def test_execute_message_payload(self):

        # Create a server that handles a request, unpacks the payload, and
        # asserts that the information in the payload was delivered successfully.
        with self.create_client(max_retries=0) as server:
            test_service = TestService(server, 1)
            server.connect()
            topic = UuidGenerator.generate_id_as_string()
            reg_info = ServiceRegistrationInfo(server, "message_payload_runner_service")

            # callback definition
            def on_request(request):

                unpacker = Unpacker(file_like=StringIO.StringIO(request.payload))

                with self.request_complete_condition:
                    try:
                        self.assertEquals(unpacker.next(), self.TEST_STRING)
                        self.assertEquals(unpacker.next(), self.TEST_BYTE)
                        self.assertEquals(unpacker.next(), self.TEST_INT)
                        self.received_request = True
                    except Exception, e:
                        print e.message
                    self.request_complete_condition.notify_all()

            request_callback = RequestCallback()
            request_callback.on_request = on_request
            reg_info.add_topic(topic, request_callback)
            # Register the service
            server.register_service_sync(reg_info, self.DEFAULT_TIMEOUT)

            with self.create_client() as client:
                client.connect()
                packer = Packer()

                # Send a request to the server with information contained
                # in the payload
                request = Request(destination_topic=topic)
                request.payload = packer.pack(self.TEST_STRING)
                request.payload += packer.pack(self.TEST_BYTE)
                request.payload += packer.pack(self.TEST_INT)
                client.async_request(request, request_callback)

                with self.request_complete_condition:
                    if not self.received_request:
                        # Wait until the request has been processed
                        self.request_complete_condition.wait(self.MAX_WAIT)
                        if not self.received_request:
                            self.fail("Request not received.")



# This sample demonstrates invoking the McAfee Threat Intelligence Exchange
# (TIE) DXL service to retrieve the reputation of files (as identified
# by their hashes)

import logging
import os
import sys
import json
import base64

from dxlclient.client import DxlClient
from dxlclient.client_config import DxlClientConfig
from dxlclient.message import Message, Request

# Import common logging and configuration
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
from common import *

# Configure local logger
logging.getLogger().setLevel(logging.ERROR)
logger = logging.getLogger(__name__)

# The topic for requesting file reputations
FILE_REP_TOPIC = "/mcafee/service/tie/file/reputation"

# Create DXL configuration from file
config = DxlClientConfig.create_dxl_config_from_file(CONFIG_FILE)


def base64_from_hex(hexstr):
    """
    Returns the base64 string for the hex string specified

    :param hexstr: The hex string to convert to base64
    :return: The base64 value for the specified hes string
    """
    return base64.b64encode(hexstr.decode('hex'))


def get_tie_file_reputation(client, md5_hex, sha1_hex):
    """
    Returns a dictionary containing the results of a TIE file reputation request

    :param client: The DXL client
    :param md5_hex: The MD5 Hex string for the file
    :param sha1_hex: The SHA-1 Hex string for the file
    :return: A dictionary containing the results of a TIE file reputation request
    """
    # Create the request message
    req = Request(FILE_REP_TOPIC)

    # Create a dictionary for the payload
    payload_dict = {
        "hashes": [
            {"type": "md5", "value": base64_from_hex(md5_hex)},
            {"type": "sha1", "value": base64_from_hex(sha1_hex)}
        ]
    }

    # Set the payload
    req.payload = json.dumps(payload_dict).encode()

    # Send the request and wait for a response (synchronous)
    res = client.sync_request(req)

    # Return a dictionary corresponding to the response payload
    if res.message_type != Message.MESSAGE_TYPE_ERROR:
        return json.loads(res.payload.decode(encoding="UTF-8"))
    else:
        raise Exception("Error: " + res.error_message + " (" + str(res.error_code) + ")")

# Create the client
with DxlClient(config) as client:

    # Connect to the fabric
    client.connect()

    #
    # Request and display reputation for notepad.exe
    #
    response_dict = get_tie_file_reputation(client=client,
                                            md5_hex="f2c7bb8acc97f92e987a2d4087d021b1",
                                            sha1_hex="7eb0139d2175739b3ccb0d1110067820be6abd29")
    print "Notepad.exe reputation:"
    print json.dumps(response_dict, sort_keys=True, indent=4, separators=(',', ': ')) + "\n"

    #
    # Request and display reputation for EICAR
    #
    response_dict = get_tie_file_reputation(client=client,
                                            md5_hex="44d88612fea8a8f36de82e1278abb02f",
                                            sha1_hex="3395856ce81f2b7382dee72602f798b642f14140")
    print "EICAR reputation:"
    print json.dumps(response_dict, sort_keys=True, indent=4, separators=(',', ': '))



# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2014 McAfee Inc. - All Rights Reserved.
################################################################################

# Run with python -m unittest dxlclient.test.test_dxlclient

import unittest

import time

from dxlclient.test.base_test import BaseClientTest
from nose_parameterized import parameterized
from mock import Mock, patch
from threading import Condition

from dxlclient import DxlClient, UuidGenerator, EventCallback, Event, ServiceRegistrationInfo, Response, \
    ResponseCallback
from dxlclient import DxlUtils, WildcardCallback
from dxlclient import DxlClientConfig
from dxlclient import RequestCallback
from dxlclient import Request
from dxlclient._global_settings import *
from nose.plugins.attrib import attr

def topic_splitter(topic):
    if topic is "":
        return "#"
    splitted = topic.split("/")
    if topic[-1] != "#":
        return "/".join(splitted[:-1]) + "/#"
    else:
        if len(topic) is 2:
            return "#"
        return "/".join(splitted[:-2]) + "/#"

class WilcardPerformanceTest(BaseClientTest):

    #
    # Test to determine whether Bug 1060219 - "Performance of Broker degrades when subscribing to channel
    # (with a lot of subscriptions) using wildcards" has been fixed.
    #
    @attr('system')
    def test_wildcard_performance(self):
        with self.create_client() as client:
            client.connect()

            without_wildcard = self.measure_performance(client, True, False)
            with_wildcard =self.measure_performance(client, False, True)
            with_wildcard_topic_exists = self.measure_performance(client, True, True)

            print "without_wildcard: " + str(without_wildcard)
            print "with_wildcard: " + str(with_wildcard)
            print "with_wildcard_topic_exists: " + str(with_wildcard_topic_exists)

            self.assertTrue(with_wildcard < (2 * without_wildcard))

    #
    # Worker associated with test to determine whether Bug 1060219 - "Performance of Broker degrades when
    # subscribing to channel (with a lot of subscriptions) using wildcards" has been fixed.
    #
    def measure_performance(self, client, with_wildcard, topic_exists):
        SUB_COUNT = 10000
        QUERY_MULTIPLIER = 10
        TOPIC_PREFIX = "/topic/" + UuidGenerator.generate_id_as_string() + "/"
        event_count = [0]
        message_ids = set()
        PAYLOAD = UuidGenerator.generate_id_as_string()
        message_id_condition = Condition()

        cb = EventCallback()

        def on_event(event):
            if event.payload == PAYLOAD:
                with message_id_condition:
                    event_count[0] += 1
                    message_ids.add(event.message_id)
                    message_id_condition.notify()
                    if len(message_ids) % SUB_COUNT == 0:
                        print "Messages size: " + str(len(message_ids))

        cb.on_event = on_event
        client.add_event_callback("#", cb, False)

        if with_wildcard:
            client.subscribe(TOPIC_PREFIX + "#")

        for i in range(SUB_COUNT):
            if i % 1000 == 0:
                print "Subscribed: " + str(i)
            client.subscribe(TOPIC_PREFIX + str(i))

        print "Subscribed."

        start_time = time.time()

        for j in range(SUB_COUNT * QUERY_MULTIPLIER):
            evt = Event(TOPIC_PREFIX + str(j % SUB_COUNT + (SUB_COUNT if not topic_exists else 0)))
            evt.payload = PAYLOAD
            client.send_event(evt)


        with message_id_condition:
            while len(message_ids) != SUB_COUNT * QUERY_MULTIPLIER \
                    or event_count[0] != SUB_COUNT * QUERY_MULTIPLIER * (2 if with_wildcard and topic_exists else 1):
                current_event = event_count[0]
                message_id_condition.wait(5)
                if current_event == event_count[0]:
                    self.fail("Event wait timeout")

        self.assertEquals(SUB_COUNT * QUERY_MULTIPLIER, len(message_ids))
        self.assertEquals(SUB_COUNT * QUERY_MULTIPLIER * (2 if with_wildcard and topic_exists else 1), event_count[0])

        return time.time() - start_time

    # Test service-based wilcarding
    # Test the ability to transform events into requests
    # Test wildcarding of events
    @attr('system')
    def test_wildcard_services(self):
        with self.create_client() as client:
            # The request message that the service receives
            service_request_message = []
            # The request message corresponding to the response received by the client
            client_response_message_request = []
            # The event that we received
            client_event_message = []
            # The payload that the service receives
            service_request_message_receive_payload = []

            client.connect()

            info = ServiceRegistrationInfo(client, "myWildcardService")
            meta = {}
            # Transform events mapped to "test/#/" to "request/test/..."
            meta["EventToRequestTopic"] = "/test/#"
            meta["EventToRequestPrefix"] = "/request"
            info.metadata = meta
            rcb = RequestCallback()

            def on_request(request):
                print "## Request in service: " + request.destination_topic + ", " + str(request.message_id)
                print "## Request in service - payload: " + request.payload

                service_request_message.append(request.message_id)
                service_request_message_receive_payload.append(request.payload)

                response = Response(request)
                response.payload = "Request response - Event payload: " + request.payload
                client.send_response(response)

            rcb.on_request = on_request
            info.add_topic("/request/test/#", rcb)

            client.register_service_sync(info, 10)

            evt = Event("/test/bar")

            rcb = ResponseCallback()
            def on_response(response):
                # Only handle the response corresponding to the event we sent
                if response.request_message_id == evt.message_id:
                    print "## received_response: " + response.request_message_id + ", " + response.__class__.__name__
                    print "## received_response_payload: " + response.payload
                    client_response_message_request[0] = response.request_message_id

            rcb.on_response = on_response
            client.add_response_callback("", rcb)

            ecb = EventCallback()
            def on_event(event):
                print "## received event: " + event.destination_topic + ", " + event.message_id
                client_event_message.append(event.message_id)

            ecb.on_event = on_event
            client.add_event_callback("/test/#", ecb)

            # Send our event
            print "## Sending event: " + evt.destination_topic + ", " + evt.message_id
            evt.payload = "Unit test payload"
            client.send_event(evt)

            time.sleep(10)

            # # Make sure the service received the request properly
            # self.assertEquals(evt.message_id, service_request_message[0])
            # # Make sure the service received the request payload from the event properly
            # self.assertEquals(evt.payload, service_request_message_receive_payload[0])
            # Make sure the response we received was for the request message
            # self.assertEquals(evt.message_id, client_response_message_request[0])
            # Make sure we received the correct event
            self.assertEquals(evt.message_id, client_event_message[0])


class WildcardTest(unittest.TestCase):
    def setUp(self):
        mqtt_client_patch = patch('paho.mqtt.client.Client')
        mqtt_client_patch.start()

        self.config = DxlClientConfig(broker_ca_bundle=get_ca_bundle_pem(),
                                      cert_file=get_cert_file_pem(),
                                      private_key=get_dxl_private_key(),
                                      brokers=[])

        self.req_callback = RequestCallback()
        self.req_callback.on_request = Mock()

    def tearDown(self):
        patch.stopall()

    @parameterized.expand([
        (3, "/foo/bar"),
        (4, "/foo/bar/baz"),
        (5, "/foo/bar/baz/"),
        (1, "/#"),
        (0, "#"),
        (1, "")
    ])
    def test_wildcarding_iteration(self, wildcard_number, topic):
        """
        Tests wildcarding iteration (utility method)
        """
        wildcards = []

        # define what is the callback for iteration
        def on_next_wildcard(wildcard):
            wildcards.append(wildcard)

        wildcard_callback = WildcardCallback()
        wildcard_callback.on_next_wildcard = on_next_wildcard

        DxlUtils.iterate_wildcards(wildcard_callback, topic)
        self.assertEquals(wildcard_number, len(wildcards))

        for wildcard in wildcards:
            topic = topic_splitter(topic)
            self.assertEqual(topic, wildcard)

    def test_adding_wildcarded_channel_enables_wildcarding(self):
        dxl_client = DxlClient(self.config)

        dxl_client.add_request_callback("/this/channel/has/wildcard/#", self.req_callback)
        self.assertTrue(dxl_client._request_callbacks.wildcarding_enabled)

    def test_adding_normal_channel_does_not_enable_wildcarding(self):
        dxl_client = DxlClient(self.config)

        dxl_client.add_request_callback("/this/channel/has/no/wildcard/", self.req_callback)
        self.assertFalse(dxl_client._request_callbacks.wildcarding_enabled)

    def test_removing_only_wildcarded_channel_disables_wildcarding(self):
        dxl_client = DxlClient(self.config)

        dxl_client.add_request_callback("/this/channel/has/wildcard/#", self.req_callback)
        dxl_client.add_request_callback("/this/channel/has/no/wildcard/", self.req_callback)
        self.assertTrue(dxl_client._request_callbacks.wildcarding_enabled)

        dxl_client.remove_request_callback("/this/channel/has/wildcard/#", self.req_callback)
        self.assertFalse(dxl_client._request_callbacks.wildcarding_enabled)

    def test_removing_one_of_two_wildcarded_channels_does_not_disable_wildcarding(self):
        dxl_client = DxlClient(self.config)

        dxl_client.add_request_callback("/this/channel/has/wildcard/#", self.req_callback)
        dxl_client.add_request_callback("/this/channel/has/wildcard/too/#", self.req_callback)

        dxl_client.remove_request_callback("/this/channel/has/wildcard/#", self.req_callback)
        self.assertTrue(dxl_client._request_callbacks.wildcarding_enabled)

    def test_messages_are_fired_with_wildcards_enabled(self):
        dxl_client = DxlClient(self.config)

        dxl_client.add_request_callback("/this/channel/has/wildcard/#", self.req_callback)
        dxl_client.add_request_callback("/this/channel/has/wildcard/not/", self.req_callback)
        # Create and process Request
        req = Request(destination_topic="/this/channel/has/wildcard/not/")._to_bytes()
        dxl_client._handle_message("/this/channel/has/wildcard/not/", req)
        # Check that callback was called
        self.assertEqual(self.req_callback.on_request.call_count, 2)

    def test_messages_are_correctly_fired_with_wildcards_disabled(self):
        dxl_client = DxlClient(self.config)

        dxl_client.add_request_callback("/this/channel/has/no/wildcard/", self.req_callback)
        dxl_client.add_request_callback("/this/channel/has/no/wildcard/either/", self.req_callback)
        # Create and process Request
        req = Request(destination_topic="/this/channel/has/no/wildcard/either/")._to_bytes()
        dxl_client._handle_message("/this/channel/has/no/wildcard/either/", req)
        # Check that callback was called
        self.assertEqual(self.req_callback.on_request.call_count, 1)



# This sample queries McAfee Active Response for the IP addresses of hosts
# that have an Active Response client installed.

import os
import sys
import json
import time

from dxlclient.client import DxlClient
from dxlclient.client_config import DxlClientConfig
from dxlclient.message import Message, Request

# Import common logging and configuration
sys.path.append(os.path.dirname(os.path.abspath(__file__)) + "/..")
from common import *

# Configure local logger
logging.getLogger().setLevel(logging.ERROR)
logger = logging.getLogger(__name__)

# The topic to create a search
CREATE_SEARCH_TOPIC = "/mcafee/mar/service/api/search"

# Create DXL configuration from file
config = DxlClientConfig.create_dxl_config_from_file(CONFIG_FILE)


def execute_mar_search_api(client, payload_dict):
    """
    Executes a query against the MAR search api

    :param client: The DXL client
    :param payload_dict: The payload
    :return: A dictionary containing the results of the query
    """
    # Create the request message
    req = Request(CREATE_SEARCH_TOPIC)
    # Set the payload
    req.payload = json.dumps(payload_dict).encode(encoding="UTF-8")

    # Display the request that is going to be sent
    print "Request:\n" + json.dumps(payload_dict, sort_keys=True, indent=4, separators=(',', ': '))

    # Send the request and wait for a response (synchronous)
    res = client.sync_request(req, timeout=30)

    # Return a dictionary corresponding to the response payload
    if res.message_type != Message.MESSAGE_TYPE_ERROR:
        resp_dict = json.loads(res.payload.decode(encoding="UTF-8"))
        # Display the response
        print "Response:\n" + json.dumps(resp_dict, sort_keys=True, indent=4, separators=(',', ': '))
        if "code" in resp_dict:
            code = resp_dict['code']
            if code < 200 or code >= 300:
                if "body" in resp_dict and "applicationErrorList" in resp_dict["body"]:
                    error = resp_dict["body"]["applicationErrorList"][0]
                    raise Exception(error["message"] + ": " + str(error["code"]))
                elif "body" in resp_dict:
                    raise Exception(resp_dict["body"] + ": " + str(code))
                else:
                    raise Exception("Error: Received failure response code: " + str(code))
        else:
            raise Exception("Error: unable to find response code")
        return resp_dict
    else:
        raise Exception("Error: " + res.error_message + " (" + str(res.error_code) + ")")

# Create the client
with DxlClient(config) as client:

    # Connect to the fabric
    client.connect()

    # Create the search
    response_dict = execute_mar_search_api(client,
        {
            "target": "/v1/simple",
            "method": "POST",
            "parameters": {},
            "body": {
                "aggregated": True,
                "projections": [
                    {
                        "name": "HostInfo",
                        "outputs": ["ip_address"]
                    }
                ]
            }
        }
    )

    # Get the search identifier
    search_id = response_dict["body"]["id"]

    # Start the search
    execute_mar_search_api(client,
        {
            "target": "/v1/" + search_id + "/start",
            "method": "PUT",
            "parameters": {},
            "body": {}
        }
    )

    # Wait until the search finishes
    finished = False
    while not finished:
        response_dict = execute_mar_search_api(client,
            {
                "target": "/v1/" + search_id + "/status",
                "method": "GET",
                "parameters": {},
                "body": {}
            }
        )
        finished = response_dict["body"]["status"] == "FINISHED"
        if not finished:
            time.sleep(5)

    # Get the search results
    # Results limited to 10, the API does support paging
    response_dict = execute_mar_search_api(client,
        {
            "target": "/v1/" + search_id + "/results",
            "method": "GET",
            "parameters": {
                "$offset": 0,
                "$limit": 10,
                "filter": "",
                "sortBy": "count",
                "sortDirection": "desc"
            },
            "body": {}
        }
    )

    # Loop and display the results
    print "Results:"
    for result in response_dict['body']['items']:
        print "    " + result['output']['HostInfo|ip_address']


# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2017 McAfee Inc. - All Rights Reserved.
################################################################################

# In order to bundle our own version of some libraries, we use the _vendor
# folder as an immutable source of external dependencies
import os
import sys
import logging
from threading import RLock

# Insert _vendor folder as the first folder when looking a library
sys.path.insert(0, os.path.join(os.path.dirname(os.path.abspath(__file__)), '_vendor'))

from dxlclient._global_settings import *
from dxlclient._product_props import *

__version__ = get_product_version()

class _NullHandler(logging.Handler):
    def emit(self, record):
        pass

logging.getLogger(__name__).addHandler(_NullHandler())

class _ObjectTracker(object):
    """
    Utility class used to track DXL Client-specific object instances
    """

    # The object tracker instance (singleton)
    _instance = None

    def __init__(self):
        """Constructor"""
        self._obj_count = 0
        self._enabled = False
        self._lock = RLock()
        self._logger = logging.getLogger(__name__)

    @property
    def enabled(self):
        """
        Whether the object tracker is enabled
        """
        return self._enabled

    @enabled.setter
    def enabled(self, val):
        self._enabled = val

    def obj_constructed(self, object):
        """
        Tracks that the specified object was constructed

        :param object: The object that was constructed
        """
        if self._enabled:
            with self._lock:
                self._obj_count += 1
                self._logger.debug("Constructed: " + object.__module__ + "." + object.__class__.__name__ \
                      + " objCount=" + str(self._obj_count))

    def obj_destructed(self, object):
        """
        Tracks that the specified object was destructed

        :param object: The object that was destructed
        """
        if self._enabled:
            with self._lock:
                self._obj_count -= 1
                self._logger.debug("Destructed: " + object.__module__ + "." + object.__class__.__name__ \
                    + " objCount=" + str(self._obj_count))

    @property
    def obj_count(self):
        """
        The current count of object instances
        """
        with self._lock:
            return self._obj_count

    @staticmethod
    def get_instance():
        """
        Returns the object tracker instance

        :return: The object tracker instance
        """
        # Instance creation should be synchronized
        if not _ObjectTracker._instance:
            _ObjectTracker._instance = _ObjectTracker()

        return _ObjectTracker._instance


class _BaseObject(object):
    """
    Base class for the DXL client-related classes
    """

    def __init__(self):
        """Constructor"""
        _ObjectTracker.get_instance().obj_constructed(self)

    def __del__(self):
        """Destructor"""
        _ObjectTracker.get_instance().obj_destructed(self)

# make all classes accessible from dxlclient package
from dxlclient._uuid_generator import *
from dxlclient.broker import *
from dxlclient._request_manager import *
from dxlclient.message import *

from dxlclient.exceptions import *
from dxlclient.callbacks import *
from dxlclient._callback_manager import *

from dxlclient.client_config import *
from dxlclient.client import *
from dxlclient._dxl_utils import *
from dxlclient.service import *


